USE [BaselineDB]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_NAME = 'sp_CollectConfigData')
	EXEC ('CREATE PROC dbo.sp_CollectConfigData AS SELECT ''stub version, to be replaced''')
GO

ALTER PROC [dbo].[sp_CollectConfigData]

/*********************************************************************************************
Written by Robert Virag

File: sp_CollectConfigData.sql

Example: 
EXEC sp_CollectConfigData
@Retention = NULL,
@BypassNonActiveSrvConfError = 0,
@help = 0,
@LogInfo = 1

Version: V1.2 - 2015.05

Tested: SQL Server 2005 / 2008 / 2008 R2 / 2012 / 2014 

Summary:
Collecting SQL Server Server Configuration Data

License: Copyright (c) 2014 Robert Virag | SQLApprentice.net

The solution is free: http://www.sqlapprentice.net/license/

*********************************************************************************************/

@Retention SMALLINT = NULL,
@BypassNonActiveSrvConfError BIT = 0,
@debug TINYINT = 0,
@help BIT = 0,
@LogInfo BIT = 1

AS
BEGIN
SET NOCOUNT ON

IF(@help = 1)
BEGIN
	SELECT (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID)) AS [Collector]
	
	SELECT N'@help' AS [Parameter Name],
           N'BIT' AS [Data Type],
		   N'0' AS [Default Value],
           N'Display help messages.' AS [Parameter Description]

    UNION ALL
	SELECT N'@LogInfo', N'BIT', N'1', N'Generate Log Information'

	UNION ALL
	SELECT N'@Retention', N'SMALLINT', N'90', N'Specify the time, in days, after which the data collected are deleted. If no time is specified, then no data are deleted.'

	UNION ALL
	SELECT N'@BypassNonActiveSrvConfError', N'BIT', N'0', N'The collector checks if there is any non-active server configuration and generates an ERROR if it finds any. If this option is enabled it bypasses the ERROR and generates a WARNING instead.'

	RETURN
END

DECLARE @ErrorMessage NVARCHAR(max)
DECLARE @InfoMessage NVARCHAR(max)
DECLARE @Severity INT

DECLARE @row_count INT
DECLARE @non_active_server_config_count INT

BEGIN TRY
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Start
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		+ N'Server: ' + CAST(SERVERPROPERTY('ServerName') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Version: ' + CAST(SERVERPROPERTY('ProductVersion') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Edition: ' + CAST(SERVERPROPERTY('Edition') AS nvarchar)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Check input parameters
	---------------------------------------------------------------------------------------------------------------------------------------
	IF @Retention < 0  
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @Retention is not supported.'
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	
	IF @BypassNonActiveSrvConfError IS NULL 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @BypassNonActiveSrvConfError is not supported.'
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	
	IF @help IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @help is not supported.'
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @LogInfo IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @LogInfo is not supported.'
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @debug IS NULL OR @debug < 0
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @debug is not supported.'
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Command
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Command Executed: EXEC ' + (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID))
		+ N' @Retention = ' + ISNULL(CAST(@Retention AS nvarchar), 'NULL')
		+ N', @BypassNonActiveSrvConfError = ' + CAST(@BypassNonActiveSrvConfError AS nvarchar)
		+ N', @LogInfo = '+ CAST(@LogInfo AS nvarchar)
		+ N', @help = ' + CAST(@help AS nvarchar) 
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Schema check
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(OBJECT_ID(N'[dbo].[configuration_data]') IS NULL)
	BEGIN
		SET @ErrorMessage = N'Table [dbo].[configuration_data] does not exist!'
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END
		
	SET @row_count = 0;

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Pre-Initial Data: insert data which have no initial state (e.g.: table is empty or data were deleted manually since the last check)
	---------------------------------------------------------------------------------------------------------------------------------------
	WITH most_recent_config as ( 
	SELECT [capture_date], [configuration_id]--, [Value], [value_in_use], [is_initial], [is_pre]
	FROM [dbo].[configuration_data] AS [c] WHERE [capture_date] = (SELECT MAX([capture_date]) FROM [dbo].[configuration_data] AS [cc] WHERE [c].[configuration_id] = [cc].[configuration_id])
	) 
	INSERT INTO [dbo].[configuration_data]
    ( [capture_date], [configuration_id], [Value], [value_in_use], [is_initial], [is_pre] ) 
	SELECT  GETDATE(),[configuration_id], [value], [value_in_use], 1, 1 
	FROM [sys].[configurations] WHERE [configuration_id] NOT IN (SELECT [configuration_id] FROM most_recent_config) 
	
	SET @row_count = @@ROWCOUNT

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) added as pre-initial config data.'
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END;

	---------------------------------------------------------------------------------------------------------
	-- Check and collect changed configuration data
	---------------------------------------------------------------------------------------------------------
	WITH [most_recent_config] AS ( 
	SELECT [capture_date], [configuration_id], [Value], [value_in_use]
	FROM [dbo].[configuration_data] AS [c] WHERE [capture_date] = (SELECT MAX([capture_date]) FROM [dbo].[configuration_data] AS [cc] WHERE [c].[configuration_id] = [cc].[configuration_id])
	)
	INSERT  INTO [dbo].[configuration_data] ( [capture_date], [configuration_id], [Value], [value_in_use], [is_initial], [is_pre] )
	SELECT GETDATE(), [c].[configuration_id], [c].[value], [c].[value_in_use], 0, 0 FROM [most_recent_config] AS [mrc] INNER JOIN [sys].[configurations] AS [c] ON [mrc].[configuration_id] = [c].[configuration_id] WHERE [mrc].[value] <> [c].[value] OR [mrc].[value_in_use] <> [c].[value_in_use] 

	SET @row_count = @@ROWCOUNT

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) added as changed config data.'
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	---------------------------------------------------------------------------------------------------------
	-- Delete expired data
	---------------------------------------------------------------------------------------------------------
	IF(@Retention > 0)
	BEGIN
		DELETE FROM [dbo].[configuration_data] WHERE [capture_date] < DATEADD(dd, -1*CAST(@retention AS nvarchar) , GETDATE())

		SET @row_count = @@ROWCOUNT

		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) deleted due to expired retention.'				
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

		---------------------------------------------------------------------------------------------------------
		-- Post-Initial Data: insert data which have no initial state because of the previous expired-deletion
		---------------------------------------------------------------------------------------------------------
		INSERT INTO [dbo].[configuration_data]
		( [capture_date], [configuration_id], [Value], [value_in_use], [is_initial], [is_pre]  )
		SELECT GETDATE(), [configuration_id], [value], [value_in_use], 1, 0 
		FROM [sys].[configurations] 
		WHERE [configuration_id] NOT IN ( 
		SELECT [configuration_id] FROM [dbo].[configuration_data] 
		WHERE [capture_date] >= DATEADD(dd, -1*@retention, GETDATE())
		)

		SET @row_count = @@ROWCOUNT

		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) added as post-initial config data.'				
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
	END;

	---------------------------------------------------------------------------------------------------------
	-- Check Non-Active Server Configurations
	---------------------------------------------------------------------------------------------------------
	WITH [most_recent_config] AS (
	SELECT [capture_date], [configuration_id], [Value], [value_in_use], [is_initial]
	FROM [dbo].[configuration_data] AS [c] WHERE [capture_date] = (SELECT MAX([capture_date]) FROM [dbo].[configuration_data] AS [cc] WHERE [c].[configuration_id] = [cc].[configuration_id])
	)
	SELECT @non_active_server_config_count = COUNT(*) FROM [most_recent_config] WHERE [Value] <> [value_in_use]
	
	IF(@non_active_server_config_count > 0)
	BEGIN
		SET @InfoMessage = N'WARNING - Non-Active Server Configuration: ' + CAST(@non_active_server_config_count AS NVARCHAR) 

		IF(@BypassNonActiveSrvConfError = 1)
		BEGIN
			SET @Severity = 10
		END
		ELSE
		BEGIN
			SET @Severity = 16
		END

		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
	
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Succeeded' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

END TRY
---------------------------------------------------------------------------------------------------------------------------------------
-- Handle Errors
---------------------------------------------------------------------------------------------------------------------------------------
BEGIN CATCH
	DECLARE @ErrMsg NVARCHAR(4000)
	DECLARE @ErrSev INT
	SET @ErrSev = ERROR_SEVERITY()
	SET @ErrMsg = ERROR_MESSAGE()

	RAISERROR(@ErrMsg, @ErrSev, 1) WITH NOWAIT
	
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Failed' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
END CATCH
END
GO
