USE [BaselineDB]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_NAME = 'sp_CollectDatabaseInfo')
	EXEC ('CREATE PROC dbo.sp_CollectDatabaseInfo AS SELECT ''stub version, to be replaced''')
GO

ALTER PROC [dbo].[sp_CollectDatabaseInfo]

/*********************************************************************************************
Written by Robert Virag

File: sp_CollectDatabaseInfo.sql

Example: 
EXEC sp_CollectDatabaseInfo
@Retention = NULL,
@help = 0,
@LogInfo = 1

Version: V1.2 - 2015.05

Tested: SQL Server 2005 / 2008 / 2008 R2 / 2012 / 2014   

Summary:
Collecting Database Info

License: Copyright (c) 2014 Robert Virag | SQLApprentice.net

The solution is free: http://www.sqlapprentice.net/license/

*********************************************************************************************/

@Retention SMALLINT = NULL,
@debug TINYINT = 0,
@help BIT = 0,
@LogInfo BIT = 1

AS
BEGIN
SET NOCOUNT ON

IF(@help = 1)
BEGIN
	SELECT (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID)) AS [Collector]
	
	SELECT N'@help' AS [Parameter Name],
           N'BIT' AS [Data Type],
		   N'0' AS [Default Value],
           N'Display help messages.' AS [Parameter Description]

    UNION ALL
	SELECT N'@LogInfo', N'BIT', N'1', N'Generate Log Information'

	UNION ALL
	SELECT N'@Retention', N'SMALLINT', N'90', N'Specify the time, in days, after which the data collected are deleted. If no time is specified, then no data are deleted.'
	RETURN
END

DECLARE @ErrorMessage NVARCHAR(max)
DECLARE @InfoMessage NVARCHAR(max)
DECLARE @Severity INT

DECLARE @SQLCommand NVARCHAR(max)
DECLARE @version_column_list NVARCHAR(max)
DECLARE @version_column_list_with_table_prefix NVARCHAR(max)
DECLARE @version_where_predicates NVARCHAR(max)

DECLARE @row_count INT

BEGIN TRY
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Start
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		+ N'Server: ' + CAST(SERVERPROPERTY('ServerName') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Version: ' + CAST(SERVERPROPERTY('ProductVersion') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Edition: ' + CAST(SERVERPROPERTY('Edition') AS nvarchar)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Check input parameters
	---------------------------------------------------------------------------------------------------------------------------------------
	IF @Retention < 0
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @Retention is not supported.'
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	
	IF @help IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @help is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @LogInfo IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @LogInfo is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @debug IS NULL OR @debug < 0
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @debug is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Command
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Command Executed: EXEC ' + (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID))
		+ N' @Retention = ' + ISNULL(CAST(@Retention AS nvarchar), N'NULL')
		+ N', @LogInfo = '+ CAST(@LogInfo AS nvarchar)
		+ N', @help = ' + CAST(@help AS nvarchar)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Schema check
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(OBJECT_ID(N'[dbo].[database_info]') IS NULL)
	BEGIN
		SET @ErrorMessage = N'Table [dbo].[database_info] does not exist!' 
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END
		
	SET @row_count = 0;
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Setting variables based on versions
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@@VERSION LIKE N'%Microsoft SQL Server 2005%')
	BEGIN
		SET @version_column_list = N''
		SET @version_column_list_with_table_prefix = N''
		SET @version_where_predicates = N''
	END
	IF(@@VERSION LIKE N'%Microsoft SQL Server 2008%')
	BEGIN
		SET @version_column_list = N', /* sql 2008+ */ [is_cdc_enabled], [is_encrypted], [is_honor_broker_priority_on] '
		SET @version_column_list_with_table_prefix  = N', /* sql 2008+ */ [sd].[is_cdc_enabled], [sd].[is_encrypted], [sd].[is_honor_broker_priority_on] '
		SET @version_where_predicates = N' OR
		 /* sql 2008+ */ [mrdbi].[is_cdc_enabled] <> [sd].[is_cdc_enabled] OR 
		 [mrdbi].[is_encrypted] <> [sd].[is_encrypted] OR 
		 [mrdbi].[is_honor_broker_priority_on] <> [sd].[is_honor_broker_priority_on] '
	END
	IF(@@VERSION LIKE N'%Microsoft SQL Server 2012%')
	BEGIN
		SET @version_column_list = N', /* sql 2008+ */ [is_cdc_enabled], [is_encrypted], [is_honor_broker_priority_on], /* sql 2012+ */ [replica_id], [group_database_id], [default_language_lcid], [default_fulltext_language_lcid], [is_nested_triggers_on], [is_transform_noise_words_on], [two_digit_year_cutoff], [containment], [target_recovery_time_in_seconds] '
		SET @version_column_list_with_table_prefix  = N', /* sql 2008+ */ [sd].[is_cdc_enabled], [sd].[is_encrypted], [sd].[is_honor_broker_priority_on], /* sql 2012+ */ [sd].[replica_id], [sd].[group_database_id], [sd].[default_language_lcid], [sd].[default_fulltext_language_lcid], [sd].[is_nested_triggers_on], [sd].[is_transform_noise_words_on], [sd].[two_digit_year_cutoff], [sd].[containment], [sd].[target_recovery_time_in_seconds] '
		SET @version_where_predicates = N' OR
		 /* sql 2008+ */ [mrdbi].[is_cdc_enabled] <> [sd].[is_cdc_enabled] OR 
		 [mrdbi].[is_encrypted] <> [sd].[is_encrypted] OR 
		 [mrdbi].[is_honor_broker_priority_on] <> [sd].[is_honor_broker_priority_on] OR 
		 /* sql 2012+ */ [sd].[replica_id] <> [sd].[replica_id] OR 
		 [mrdbi].[group_database_id] <> [sd].[group_database_id] OR 
		 [mrdbi].[default_language_lcid] <> [sd].[default_language_lcid] OR 
		 [mrdbi].[default_fulltext_language_lcid] <> [sd].[default_fulltext_language_lcid] OR 
		 [mrdbi].[is_nested_triggers_on] <> [sd].[is_nested_triggers_on] OR 
		 [mrdbi].[is_transform_noise_words_on] <> [sd].[is_transform_noise_words_on] OR 
		 [mrdbi].[two_digit_year_cutoff] <> [sd].[two_digit_year_cutoff] OR 
		 [mrdbi].[containment] <> [sd].[containment] OR 
		 [mrdbi].[target_recovery_time_in_seconds] <> [sd].[target_recovery_time_in_seconds] '
	END
	IF(@@VERSION LIKE N'%Microsoft SQL Server 2014%')
	BEGIN
		SET @version_column_list = N', /* sql 2008+ */ [is_cdc_enabled], [is_encrypted], [is_honor_broker_priority_on], /* sql 2012+ */ [replica_id], [group_database_id], [default_language_lcid], [default_fulltext_language_lcid], [is_nested_triggers_on], [is_transform_noise_words_on], [two_digit_year_cutoff], [containment], [target_recovery_time_in_seconds], /* sql 2014 */ [is_auto_create_stats_incremental_on], [is_query_store_on], [resource_pool_id], [delayed_durability], [is_memory_optimized_elevate_to_snapshot_on] '
		SET @version_column_list_with_table_prefix  = N', /* sql 2008+ */ [sd].[is_cdc_enabled], [sd].[is_encrypted], [sd].[is_honor_broker_priority_on], /* sql 2012+ */ [sd].[replica_id], [sd].[group_database_id], [sd].[default_language_lcid], [sd].[default_fulltext_language_lcid], [sd].[is_nested_triggers_on], [sd].[is_transform_noise_words_on], [sd].[two_digit_year_cutoff], [sd].[containment], [sd].[target_recovery_time_in_seconds], /* sql 2014 */ [sd].[is_auto_create_stats_incremental_on], [sd].[is_query_store_on], [sd].[resource_pool_id], [sd].[delayed_durability], [sd].[is_memory_optimized_elevate_to_snapshot_on] '
		SET @version_where_predicates = N' OR
		 /* sql 2008+ */ [mrdbi].[is_cdc_enabled] <> [sd].[is_cdc_enabled] OR 
		 [mrdbi].[is_encrypted] <> [sd].[is_encrypted] OR 
		 [mrdbi].[is_honor_broker_priority_on] <> [sd].[is_honor_broker_priority_on] OR 
		 /* sql 2012+ */ [mrdbi].[replica_id] <> [sd].[replica_id] OR 
		 [mrdbi].[group_database_id] <> [sd].[group_database_id] OR 
		 [mrdbi].[default_language_lcid] <> [sd].[default_language_lcid] OR 
		 [mrdbi].[default_fulltext_language_lcid] <> [sd].[default_fulltext_language_lcid] OR 
		 [mrdbi].[is_nested_triggers_on] <> [sd].[is_nested_triggers_on] OR 
		 [mrdbi].[is_transform_noise_words_on] <> [sd].[is_transform_noise_words_on] OR 
		 [mrdbi].[two_digit_year_cutoff] <> [sd].[two_digit_year_cutoff] OR 
		 [mrdbi].[containment] <> [sd].[containment] OR 
		 [mrdbi].[target_recovery_time_in_seconds] <> [sd].[target_recovery_time_in_seconds] OR
		 /* sql 2014 */ [mrdbi].[is_auto_create_stats_incremental_on] <> [sd].[is_auto_create_stats_incremental_on] OR
		 [mrdbi].[is_query_store_on] <> [sd].[is_query_store_on] OR
		 [mrdbi].[resource_pool_id] <> [sd].[resource_pool_id] OR
		 [mrdbi].[delayed_durability] <> [sd].[delayed_durability] OR
		 [mrdbi].[is_memory_optimized_elevate_to_snapshot_on] <> [sd].[is_memory_optimized_elevate_to_snapshot_on] '
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Pre-Initial Data: insert data which have no initial state (e.g.: table is empty or data were deleted manually since the last check)
	---------------------------------------------------------------------------------------------------------------------------------------
	SET @SQLCommand = N'WITH most_recent_db_info as ( 
		 SELECT [capture_date], [database_name] 
		 FROM [dbo].[database_info] AS [di] WHERE [capture_date] = (SELECT MAX([capture_date]) FROM [dbo].[database_info] AS [di2] WHERE [di].[database_name] = [di2].[database_name]) 
		 ) 
		 INSERT INTO [dbo].[database_info] 
		 ( [capture_date], [database_name], [database_id], [source_database_id], [owner_sid], [create_date], [compatibility_level], [collation_name], [user_access], [is_read_only], [is_auto_close_on], [is_auto_shrink_on], [state], [is_in_standby], [is_supplemental_logging_enabled], [snapshot_isolation_state], [is_read_committed_snapshot_on], [recovery_model], [page_verify_option], [is_auto_create_stats_on], [is_auto_update_stats_on], [is_auto_update_stats_async_on], [is_ansi_null_default_on], [is_ansi_nulls_on], [is_ansi_padding_on], [is_ansi_warnings_on], [is_arithabort_on], [is_concat_null_yields_null_on], [is_numeric_roundabort_on], [is_quoted_identifier_on], [is_recursive_triggers_on], [is_cursor_close_on_commit_on], [is_local_cursor_default], [is_fulltext_enabled], [is_trustworthy_on], [is_db_chaining_on], [is_parameterization_forced], [is_master_key_encrypted_by_server], [is_published], [is_merge_published], [is_distributor], [is_sync_with_backup], [service_broker_guid], [is_broker_enabled], [is_date_correlation_on] ' + @version_column_list + N', [is_initial], [is_pre] ) 
		 SELECT  GETDATE(),[name], [database_id], [source_database_id], [owner_sid], [create_date], [compatibility_level], [collation_name], [user_access], [is_read_only], [is_auto_close_on], [is_auto_shrink_on], [state], [is_in_standby], [is_supplemental_logging_enabled], [snapshot_isolation_state], [is_read_committed_snapshot_on], [recovery_model], [page_verify_option], [is_auto_create_stats_on], [is_auto_update_stats_on], [is_auto_update_stats_async_on], [is_ansi_null_default_on], [is_ansi_nulls_on], [is_ansi_padding_on], [is_ansi_warnings_on], [is_arithabort_on], [is_concat_null_yields_null_on], [is_numeric_roundabort_on], [is_quoted_identifier_on], [is_recursive_triggers_on], [is_cursor_close_on_commit_on], [is_local_cursor_default], [is_fulltext_enabled], [is_trustworthy_on], [is_db_chaining_on], [is_parameterization_forced], [is_master_key_encrypted_by_server], [is_published], [is_merge_published], [is_distributor], [is_sync_with_backup], [service_broker_guid], [is_broker_enabled], [is_date_correlation_on] ' + @version_column_list + N', 1, 1  
		 FROM [sys].[databases] WHERE [name] NOT IN (SELECT [database_name] FROM most_recent_db_info) '
	EXEC sp_executesql @SQLCommand

	SET @row_count = @@ROWCOUNT

	
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) added as pre-initial config data.' 
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END;

	---------------------------------------------------------------------------------------------------------
	-- Check and collect changed configuration data
	---------------------------------------------------------------------------------------------------------
	SET @SQLCommand = N'WITH most_recent_db_info as ( 
		SELECT [capture_date], [database_name], [database_id], [source_database_id], [owner_sid], [create_date], [compatibility_level], [collation_name], [user_access], [is_read_only], [is_auto_close_on], [is_auto_shrink_on], [state], [is_in_standby], [is_supplemental_logging_enabled], [snapshot_isolation_state], [is_read_committed_snapshot_on], [recovery_model], [page_verify_option], [is_auto_create_stats_on], [is_auto_update_stats_on], [is_auto_update_stats_async_on], [is_ansi_null_default_on], [is_ansi_nulls_on], [is_ansi_padding_on], [is_ansi_warnings_on], [is_arithabort_on], [is_concat_null_yields_null_on], [is_numeric_roundabort_on], [is_quoted_identifier_on], [is_recursive_triggers_on], [is_cursor_close_on_commit_on], [is_local_cursor_default], [is_fulltext_enabled], [is_trustworthy_on], [is_db_chaining_on], [is_parameterization_forced], [is_master_key_encrypted_by_server], [is_published], [is_merge_published], [is_distributor], [is_sync_with_backup], [service_broker_guid], [is_broker_enabled], [is_date_correlation_on] ' + @version_column_list + N' 
		FROM [dbo].[database_info] AS [di] WHERE [capture_date] = (SELECT MAX([capture_date]) FROM [dbo].[database_info] AS [di2] WHERE [di].[database_name] = [di2].[database_name])
		) 
		INSERT  INTO [dbo].[database_info] ( [capture_date], [database_name], [database_id], [source_database_id], [owner_sid], [create_date], [compatibility_level], [collation_name], [user_access], [is_read_only], [is_auto_close_on], [is_auto_shrink_on], [state], [is_in_standby], [is_supplemental_logging_enabled], [snapshot_isolation_state], [is_read_committed_snapshot_on], [recovery_model], [page_verify_option], [is_auto_create_stats_on], [is_auto_update_stats_on], [is_auto_update_stats_async_on], [is_ansi_null_default_on], [is_ansi_nulls_on], [is_ansi_padding_on], [is_ansi_warnings_on], [is_arithabort_on], [is_concat_null_yields_null_on], [is_numeric_roundabort_on], [is_quoted_identifier_on], [is_recursive_triggers_on], [is_cursor_close_on_commit_on], [is_local_cursor_default], [is_fulltext_enabled], [is_trustworthy_on], [is_db_chaining_on], [is_parameterization_forced], [is_master_key_encrypted_by_server], [is_published], [is_merge_published], [is_distributor], [is_sync_with_backup], [service_broker_guid], [is_broker_enabled], [is_date_correlation_on] ' + @version_column_list + N', [is_initial], [is_pre] )
		SELECT GETDATE(), [sd].[name], [sd].[database_id], [sd].[source_database_id], [sd].[owner_sid], [sd].[create_date], [sd].[compatibility_level], [sd].[collation_name], [sd].[user_access], [sd].[is_read_only], [sd].[is_auto_close_on], [sd].[is_auto_shrink_on], [sd].[state], [sd].[is_in_standby], [sd].[is_supplemental_logging_enabled], [sd].[snapshot_isolation_state], [sd].[is_read_committed_snapshot_on], [sd].[recovery_model], [sd].[page_verify_option], [sd].[is_auto_create_stats_on], [sd].[is_auto_update_stats_on], [sd].[is_auto_update_stats_async_on], [sd].[is_ansi_null_default_on], [sd].[is_ansi_nulls_on], [sd].[is_ansi_padding_on], [sd].[is_ansi_warnings_on], [sd].[is_arithabort_on], [sd].[is_concat_null_yields_null_on], [sd].[is_numeric_roundabort_on], [sd].[is_quoted_identifier_on], [sd].[is_recursive_triggers_on], [sd].[is_cursor_close_on_commit_on], [sd].[is_local_cursor_default], [sd].[is_fulltext_enabled], [sd].[is_trustworthy_on], [sd].[is_db_chaining_on], [sd].[is_parameterization_forced], [sd].[is_master_key_encrypted_by_server], [sd].[is_published], [sd].[is_merge_published], [sd].[is_distributor], [sd].[is_sync_with_backup], [sd].[service_broker_guid], [sd].[is_broker_enabled], [sd].[is_date_correlation_on] ' + @version_column_list_with_table_prefix + N', 0, 0 
		FROM [most_recent_db_info] AS [mrdbi] INNER JOIN [sys].[databases] AS [sd] ON [mrdbi].[database_name] = [sd].[name] 
		WHERE [mrdbi].[database_id] <> [sd].[database_id] OR  
		 [mrdbi].[source_database_id] <> [sd].[source_database_id] OR 
		 [mrdbi].[owner_sid] <> [sd].[owner_sid] OR 
		 [mrdbi].[create_date] <> [sd].[create_date] OR 
		 [mrdbi].[compatibility_level] <> [sd].[compatibility_level] OR 
		 [mrdbi].[collation_name] <> [sd].[collation_name] OR 
		 [mrdbi].[user_access] <> [sd].[user_access] OR 
		 [mrdbi].[is_read_only] <> [sd].[is_read_only] OR 
		 [mrdbi].[is_auto_close_on] <> [sd].[is_auto_close_on] OR 
		 [mrdbi].[is_auto_shrink_on] <> [sd].[is_auto_shrink_on] OR 
		 [mrdbi].[state] <> [sd].[state] OR 
		 [mrdbi].[is_in_standby] <> [sd].[is_in_standby] OR 
		 [mrdbi].[is_supplemental_logging_enabled] <> [sd].[is_supplemental_logging_enabled] OR 
		 [mrdbi].[snapshot_isolation_state] <> [sd].[snapshot_isolation_state] OR 
		 [mrdbi].[is_read_committed_snapshot_on] <> [sd].[is_read_committed_snapshot_on] OR 
		 [mrdbi].[recovery_model] <> [sd].[recovery_model] OR 
		 [mrdbi].[page_verify_option] <> [sd].[page_verify_option] OR 
		 [mrdbi].[is_auto_create_stats_on] <> [sd].[is_auto_create_stats_on] OR 
		 [mrdbi].[is_auto_update_stats_on] <> [sd].[is_auto_update_stats_on] OR 
		 [mrdbi].[is_auto_update_stats_async_on] <> [sd].[is_auto_update_stats_async_on] OR 
		 [mrdbi].[is_ansi_null_default_on] <> [sd].[is_ansi_null_default_on] OR 
		 [mrdbi].[is_ansi_nulls_on] <> [sd].[is_ansi_nulls_on] OR 
		 [mrdbi].[is_ansi_padding_on] <> [sd].[is_ansi_padding_on] OR 
		 [mrdbi].[is_ansi_warnings_on] <> [sd].[is_ansi_warnings_on] OR 
		 [mrdbi].[is_arithabort_on] <> [sd].[is_arithabort_on] OR 
		 [mrdbi].[is_concat_null_yields_null_on] <> [sd].[is_concat_null_yields_null_on] OR 
		 [mrdbi].[is_numeric_roundabort_on] <> [sd].[is_numeric_roundabort_on] OR 
		 [mrdbi].[is_quoted_identifier_on] <> [sd].[is_quoted_identifier_on] OR 
		 [mrdbi].[is_recursive_triggers_on] <> [sd].[is_recursive_triggers_on] OR 
		 [mrdbi].[is_cursor_close_on_commit_on] <> [sd].[is_cursor_close_on_commit_on] OR 
		 [mrdbi].[is_local_cursor_default] <> [sd].[is_local_cursor_default] OR 
		 [mrdbi].[is_fulltext_enabled] <> [sd].[is_fulltext_enabled] OR 
		 [mrdbi].[is_trustworthy_on] <> [sd].[is_trustworthy_on] OR 
		 [mrdbi].[is_db_chaining_on] <> [sd].[is_db_chaining_on] OR 
		 [mrdbi].[is_parameterization_forced] <> [sd].[is_parameterization_forced] OR 
		 [mrdbi].[is_master_key_encrypted_by_server] <> [sd].[is_master_key_encrypted_by_server] OR 
		 [mrdbi].[is_published] <> [sd].[is_published] OR 
		 [mrdbi].[is_merge_published] <> [sd].[is_merge_published] OR 
		 [mrdbi].[is_distributor] <> [sd].[is_distributor] OR 
		 [mrdbi].[is_sync_with_backup] <> [sd].[is_sync_with_backup] OR 
		 [mrdbi].[service_broker_guid] <> [sd].[service_broker_guid] OR 
		 [mrdbi].[is_broker_enabled] <> [sd].[is_broker_enabled] OR 
		 [mrdbi].[is_date_correlation_on] <> [sd].[is_date_correlation_on] ' + @version_where_predicates
		 EXEC sp_executesql @SQLCommand

		 SET @row_count = @@ROWCOUNT

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) added as changed config data.' 
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	

	---------------------------------------------------------------------------------------------------------
	-- Delete expired data
	---------------------------------------------------------------------------------------------------------
	IF(@Retention > 0)
	BEGIN
		DELETE FROM [dbo].[database_info] WHERE [capture_date] < DATEADD(dd, -1*CAST(@retention AS nvarchar) , GETDATE())

		SET @row_count = @@ROWCOUNT

		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) deleted due to expired retention.' 				
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

		---------------------------------------------------------------------------------------------------------
		-- Post-Initial Data: insert data which have no initial state because of the previous expired-deletion
		---------------------------------------------------------------------------------------------------------
		SET @SQLCommand = N'INSERT INTO [dbo].[database_info] 
		( [capture_date], [database_name], [database_id], [source_database_id], [owner_sid], [create_date], [compatibility_level], [collation_name], [user_access], [is_read_only], [is_auto_close_on], [is_auto_shrink_on], [state], [is_in_standby], [is_supplemental_logging_enabled], [snapshot_isolation_state], [is_read_committed_snapshot_on], [recovery_model], [page_verify_option], [is_auto_create_stats_on], [is_auto_update_stats_on], [is_auto_update_stats_async_on], [is_ansi_null_default_on], [is_ansi_nulls_on], [is_ansi_padding_on], [is_ansi_warnings_on], [is_arithabort_on], [is_concat_null_yields_null_on], [is_numeric_roundabort_on], [is_quoted_identifier_on], [is_recursive_triggers_on], [is_cursor_close_on_commit_on], [is_local_cursor_default], [is_fulltext_enabled], [is_trustworthy_on], [is_db_chaining_on], [is_parameterization_forced], [is_master_key_encrypted_by_server], [is_published], [is_merge_published], [is_distributor], [is_sync_with_backup], [service_broker_guid], [is_broker_enabled], [is_date_correlation_on] ' + @version_column_list + N', [is_initial], [is_pre] ) 
		 SELECT GETDATE(), [name], [database_id], [source_database_id], [owner_sid], [create_date], [compatibility_level], [collation_name], [user_access], [is_read_only], [is_auto_close_on], [is_auto_shrink_on], [state], [is_in_standby], [is_supplemental_logging_enabled], [snapshot_isolation_state], [is_read_committed_snapshot_on], [recovery_model], [page_verify_option], [is_auto_create_stats_on], [is_auto_update_stats_on], [is_auto_update_stats_async_on], [is_ansi_null_default_on], [is_ansi_nulls_on], [is_ansi_padding_on], [is_ansi_warnings_on], [is_arithabort_on], [is_concat_null_yields_null_on], [is_numeric_roundabort_on], [is_quoted_identifier_on], [is_recursive_triggers_on], [is_cursor_close_on_commit_on], [is_local_cursor_default], [is_fulltext_enabled], [is_trustworthy_on], [is_db_chaining_on], [is_parameterization_forced], [is_master_key_encrypted_by_server], [is_published], [is_merge_published], [is_distributor], [is_sync_with_backup], [service_broker_guid], [is_broker_enabled], [is_date_correlation_on] ' + @version_column_list + N', 1, 0  
		 FROM [sys].[databases] 
		 WHERE [name] NOT IN ( 
		 SELECT [database_name] FROM [dbo].[database_info] 
		 WHERE [capture_date] >= DATEADD(dd, -1*'+CAST(@retention AS nvarchar)+N', GETDATE()) 
		 ) '
		EXEC sp_executesql @SQLCommand

		SET @row_count = @@ROWCOUNT

		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) added as post-initial config data.' 				
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
	END;


	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Succeeded' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

END TRY
---------------------------------------------------------------------------------------------------------------------------------------
-- Handle Errors
---------------------------------------------------------------------------------------------------------------------------------------
BEGIN CATCH
	DECLARE @ErrMsg NVARCHAR(4000)
	DECLARE @ErrSev INT
	SET @ErrSev = ERROR_SEVERITY()
	SET @ErrMsg = ERROR_MESSAGE()

	RAISERROR(@ErrMsg, @ErrSev, 1) WITH NOWAIT
	
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Failed' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
END CATCH
END
GO
