USE [BaselineDB]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_NAME = 'sp_CollectFileInfo')
	EXEC ('CREATE PROC dbo.sp_CollectFileInfo AS SELECT ''stub version, to be replaced''')
GO

ALTER PROC [dbo].[sp_CollectFileInfo]

/*********************************************************************************************
Written by Robert Virag

File: sp_CollectFileInfo.sql

Example:
EXEC sp_CollectFileInfo
@DestTable = NULL,
@DestSchema = NULL,
@TSMode = 0,
@EmptyTSTable = 0,
@Retention = 90,
@help = 0,
@LogInfo = 1

Version: V1.2 - 2015.05

Tested: SQL Server 2005 / 2008 / 2008 R2 / 2012 / 2014 

Summary:
Collecting SQL Server Database File Information Raw Data

License: Copyright (c) 2014 Robert Virag | SQLApprentice.net

The solution is free: http://www.sqlapprentice.net/license/

*********************************************************************************************/

@DestTable NVARCHAR(128) = NULL,
@DestSchema NVARCHAR(25) = NULL,
@TSMode BIT = 0,
@EmptyTSTable BIT = 0,
@Retention SMALLINT = 90,
@debug TINYINT = 0,
@help BIT = 0,
@LogInfo BIT = 1

AS
BEGIN

SET NOCOUNT ON

IF(@help = 1)
BEGIN
	SELECT (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID)) AS [Collector]
	
	SELECT N'@help' AS [Parameter Name],
           N'BIT' AS [Data Type],
		   N'0' AS [Default Value],
           N'Display help message.' AS [Parameter Description]

    UNION ALL
	SELECT N'@LogInfo', N'BIT', N'1', N'Generate Log Information'

	UNION ALL
	SELECT N'@Retention', N'SMALLINT', N'90', N'Specify the time, in days, after which the data collected are deleted. If no time is specified, then no data are deleted.'
	
	UNION ALL
	SELECT N'@DestTable', N'NVARCHAR(128)', N'NULL', N'In troubleshooting mode it specifies the output table. Table must exist. Can be NULL.'

	UNION ALL
	SELECT N'@DestSchema', N'NVARCHAR(25)', N'NULL', N'In troubleshooting mode it specifies the schema of the output table. Schema must exist. Can be NULL.'

	UNION ALL
	SELECT N'@TSMode', N'BIT', N'0', N'Enable troubleshooting mode'

	UNION ALL
	SELECT N'@EmptyTSTable', N'BIT', N'0', N'Empty the table used for troubleshooting before the data collection.'

	SELECT N'Standard' AS [Running Mode], N'[dbo].[file_info]' AS [Default Table]
	UNION ALL
	SELECT N'Troubleshooting', N'[dbo].[ts_file_info]'

	SELECT N'Filtering table' AS [Filtering], N'[dbo].[filter_database_file]' AS [Info]
    UNION ALL
	SELECT N'Type of Filtering', N'Exclude - Include (see the columns)'
	UNION ALL
	SELECT N'Column used in Standardmode (exclude)', N'[is_excluded_file_info]'
	UNION ALL
	SELECT N'Column used in TSMode (include)', N'[ts_is_included_file_info]'  

	SELECT [database_name], [logical_file_name], [is_excluded_file_info], [ts_is_included_file_info] FROM [dbo].[filter_database_file] ORDER BY [database_name]

	RETURN
END

DECLARE @ErrorMessage NVARCHAR(max)
DECLARE @InfoMessage NVARCHAR(max)
DECLARE @Severity INT

DECLARE @data_compression NVARCHAR(35)
DECLARE @SQLCommand NVARCHAR(max)

DECLARE @row_count INT

BEGIN TRY
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Start
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		+ N'Server: ' + CAST(SERVERPROPERTY('ServerName') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Version: ' + CAST(SERVERPROPERTY('ProductVersion') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Edition: ' + CAST(SERVERPROPERTY('Edition') AS nvarchar)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Check input parameters
	---------------------------------------------------------------------------------------------------------------------------------------
	IF @TSMode IS NULL 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @TSMode is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @EmptyTSTable IS NULL 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @EmptyTSTable is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @Retention < 0 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @Retention is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	
	IF @help IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @help is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @LogInfo IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @LogInfo is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @debug IS NULL OR @debug < 0
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @debug is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	--------------
	-- TSMode = 0
	--------------
	IF(@TSMode = 0)
	BEGIN
		SET @DestTable = N'file_info'
		SET @DestSchema = N'dbo'
	END
	--------------
	-- TSMode = 1
	--------------
	ELSE
	BEGIN
		IF(@DestTable IS NOT NULL)
		BEGIN
			SET @DestTable = REPLACE(REPLACE(RTRIM(LTRIM(@DestTable)), N'[',N''), N']',N'')
		END
		ELSE
		BEGIN
			SET @DestTAble = N'ts_file_info'
		END
		
		IF(@DestSchema IS NOT NULL)
		BEGIN
			SET @DestSchema = REPLACE(REPLACE(RTRIM(LTRIM(@DestSchema)), N'[',N''), N']',N'')
		END
		ELSE
		BEGIN
			SET @DestSchema = N'dbo'
		END
	END
	
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Command
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Command Executed: EXEC ' + (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID)) 
		+ N' @DestTable = ' + ISNULL(N''''+@DestTable+N'''', N'NULL')
		+ N', @DestSchema  = ' + ISNULL(N''''+@DestSchema+N'''', N'NULL')
		+ N', @TSMode = ' + CAST(@TSMode AS nvarchar)
		+ N', @EmptyTSTable = ' + CAST(@EmptyTSTable AS nvarchar)
		+ N', @Retention = ' + ISNULL(CAST(@Retention AS nvarchar), N'NULL')
		+ N', @LogInfo = ' + CAST(@LogInfo AS nvarchar)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	IF(@Retention > 0 AND @TSMode = 1)
	BEGIN
		SET @Retention = 0
		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = N'Parameter @Retention has been changed to 0. Reason: @TSMode is enabled. Tables for troubleshooting have to be maintained manually.' 
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
	END
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Schema check
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(OBJECT_ID(QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable)) IS NULL)
	BEGIN
		SET @ErrorMessage = N'Table ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + N' does not exist!'
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END

	IF(@TSMode = 1 AND @EmptyTSTable = 1)
	BEGIN
		SET @SQLCommand = N'TRUNCATE TABLE ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) 
		EXEC sp_executesql @SQLCommand
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Collect File Info Data 
	---------------------------------------------------------------------------------------------------------------------------------------
	CREATE TABLE #temp_filter
		(
		[database_id] INT, 
		[logical_file_id] INT, 
		)
		
	---------------------------------------------------------
	-- Filtering | TSMode - include | StandardMode - exclude
	---------------------------------------------------------
	IF(@TSMode = 1) /* INCLUDE */
	BEGIN
		INSERT INTO #temp_filter ( [database_id], [logical_file_id] )
		SELECT [sd].[database_id], [mf].[file_id] FROM [sys].[databases] AS [sd]
		INNER JOIN [sys].[master_files] AS [mf] ON [mf].[database_id] = [sd].[database_id]
		INNER JOIN [dbo].[filter_database_file] AS [ft] ON [ft].[database_name] = [sd].[name] AND [ft].[logical_file_name] = [mf].[name]
		WHERE [sd].[state] = 0 AND [ft].[ts_is_included_file_info] = 1

	END
	ELSE /* EXCLUDE */
	BEGIN
		INSERT INTO #temp_filter ( [database_id], [logical_file_id] )
		SELECT [sd].[database_id], [mf].[file_id] FROM [sys].[databases] AS [sd]
		INNER JOIN [sys].[master_files] AS [mf] ON [mf].[database_id] = [sd].[database_id]
		LEFT OUTER JOIN [dbo].[filter_database_file] AS [ft] ON [ft].[database_name] = [sd].[name] AND [ft].[logical_file_name] = [mf].[name]
		WHERE [sd].[state] = 0 AND ([ft].[is_excluded_file_info] = 0 OR [ft].[is_excluded_file_info] IS NULL)
	END

	DECLARE @object NVARCHAR (300)
	DECLARE c CURSOR LOCAL FORWARD_ONLY STATIC READ_ONLY
	FOR SELECT DISTINCT QUOTENAME(DB_NAME([database_id])) FROM #temp_filter
	OPEN c;
	
	FETCH NEXT FROM c INTO @object
	WHILE @@FETCH_STATUS = 0
	BEGIN
	SET @sqlcommand = N'USE ' + @object + N' INSERT INTO ' + QUOTENAME(DB_NAME(DB_ID()))+N'.'+QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + N' 
		 ( [capture_date], [database_name], [physical_name], [type], [size_pages], [space_used_pages], [max_size_pages], [growth_pages_percent], [is_percent_growth] ) 
		 SELECT GETDATE() AS [capture_date], DB_NAME() AS [database_name], [df].[physical_name], [df].[type], [df].[size] AS [size_pages], FILEPROPERTY([df].[name], ''SpaceUsed'') AS [space_used_pages], [df].[max_size], [df].[growth] AS [growth_pages_percent], [df].[is_percent_growth] FROM [sys].[database_files] AS [df] 
		 INNER JOIN #temp_filter AS [f] ON DB_ID() = [f].[database_id] AND [df].[file_id] = [f].[logical_file_id] '
	EXEC sp_executesql @sqlcommand
	FETCH NEXT FROM c INTO @object;
	END
	CLOSE c;
	DEALLOCATE c;

	DROP TABLE #temp_filter

	---------------------------------------------------------------------------------------------------------
	-- Delete expired data
	---------------------------------------------------------------------------------------------------------
	IF(@Retention > 0)
	BEGIN
		SET @SQLCommand = N'DELETE FROM ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + ' WHERE [capture_date] < DATEADD(dd, -1*' + CAST(@retention AS nvarchar) + N', GETDATE()) '
		EXEC sp_executesql @SQLCommand

		SET @row_count = @@ROWCOUNT

		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) deleted due to expired retention.' 				
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
	END

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = 'Outcome: Succeeded' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

END TRY
---------------------------------------------------------------------------------------------------------------------------------------
-- Handle Errors
---------------------------------------------------------------------------------------------------------------------------------------
BEGIN CATCH
	DECLARE @ErrMsg NVARCHAR(4000)
	DECLARE @ErrSev INT
	SET @ErrSev = ERROR_SEVERITY()
	SET @ErrMsg = ERROR_MESSAGE()
	
	IF OBJECT_ID ('tempdb..#temp_filter') IS NOT NULL
	BEGIN
		DROP TABLE #temp_filter
	END

	RAISERROR(@ErrMsg, @ErrSev, 1) WITH NOWAIT

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Failed' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
END CATCH
END
GO
