USE [BaselineDB]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_NAME = 'sp_CollectIOVFStats')
	EXEC ('CREATE PROC dbo.sp_CollectIOVFStats AS SELECT ''stub version, to be replaced''')
GO

ALTER PROC [dbo].[sp_CollectIOVFStats]

/*********************************************************************************************
Written by Robert Virag

File: sp_CollectIOVFStats.sql

Example:
EXEC sp_CollectIOVFStats
@DestTable = NULL,
@DestSchema  = NULL,
@TSMode = 1,
@EmptyTSTable = 1,
@CollectingInterval  = NULL,
@SampleInterval = 10,
@Retention = 90,
@help = 0,
@LogInfo = 1

Version: V1.2 - 2015.05

Tested: SQL Server 2005 / 2008 / 2008 R2 / 2012 / 2014  

Summary:
Collecting SQL Server IO Virtual File Statistics Raw Data

License: Copyright (c) 2014 Robert Virag | SQLApprentice.net

The solution is free: http://www.sqlapprentice.net/license/

*********************************************************************************************/

@DestTable NVARCHAR(128) = NULL,
@DestSchema NVARCHAR(25) = NULL,
@TSMode BIT = 0,
@EmptyTSTable BIT = 0,
@Retention SMALLINT = 90,
@CollectingInterval SMALLINT = NULL,
@SampleInterval SMALLINT = 30,
@debug TINYINT = 0,
@help BIT = 0,
@LogInfo BIT = 1

AS
BEGIN
SET NOCOUNT ON

IF(@help = 1)
BEGIN
	SELECT (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID)) AS [Collector]
	
    SELECT N'@help' AS [Parameter Name],
           N'BIT' AS [Data Type],
		   N'0' AS [Default Value],
           N'Display help message.' AS [Parameter Description]

    UNION ALL
	SELECT N'@LogInfo', N'BIT', N'1', N'Generate Log Information'

	UNION ALL
	SELECT N'@Retention', N'SMALLINT', N'90', N'Specify the time, in days, after which the data collected are deleted. If no time is specified, then no data are deleted.'
	
	UNION ALL
	SELECT N'@DestTable', N'NVARCHAR(128)', N'NULL', N'In troubleshooting mode it specifies the output table. Table must exist. Can be NULL.'

	UNION ALL
	SELECT N'@DestSchema', N'NVARCHAR(25)', N'NULL', N'In troubleshooting mode it specifies the schema of the output table. Schema must exist. Can be NULL.'

	UNION ALL
	SELECT N'@TSMode', N'BIT', N'0', N'Enable troubleshooting mode'

	UNION ALL
	SELECT N'@EmptyTSTable', N'BIT', N'0', N'Empty the table used for troubleshooting before the data collection.'

	UNION ALL
	SELECT N'@CollectingInterval', N'SMALLINT', N'NULL', N'Duration of the data collection in minutes. Maximum 1440 (1 day), so value has to be between 1 and 1440. It can be NULL. In that case the collector only takes one sample, the actual state of the data collected by the collector.'

	UNION ALL
	SELECT N'@SampleInterval', N'SMALLINT', N'30', N'Time in seconds between taking two samples. (1-3600)'
	
	SELECT N'Standard' AS [Running Mode], N'[dbo].[iovf_stats]' AS [Default Table]
	UNION ALL
	SELECT N'Troubleshooting', N'[dbo].[ts_iovf_stats]'

	SELECT N'Filtering table' AS [Filtering], N'[dbo].[filter_database_file]' AS [Info]
    UNION ALL
	SELECT N'Type of Filtering', N'Exclude - Include (see the columns)'
	UNION ALL
	SELECT N'Column used in Standardmode (exclude)', N'[is_excluded_iovf]'
	UNION ALL
	SELECT N'Column used in TSMode (include)', N'[ts_is_included_iovf]'  

	SELECT [database_name], [logical_file_name], [is_excluded_iovf], [ts_is_included_iovf] FROM [dbo].[filter_database_file] ORDER BY [database_name]

	RETURN
END

DECLARE @ErrorMessage NVARCHAR(max)
DECLARE @InfoMessage NVARCHAR(max)
DECLARE @Severity INT

DECLARE @SQLCommand NVARCHAR(max)
DECLARE @CollectSQLCommand NVARCHAR(max)
DECLARE @sample_interval NVARCHAR(12)

DECLARE @start_loop DATETIME
DECLARE @max_loop INT
DECLARE @loop_count INT

DECLARE @LockResult INT
DECLARE @RequiredAppLock NVARCHAR(25)

DECLARE @row_count INT

BEGIN TRY
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Start
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
	SET @InfoMessage = N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		+ N'Server: ' + CAST(SERVERPROPERTY('ServerName') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Version: ' + CAST(SERVERPROPERTY('ProductVersion') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Edition: ' + CAST(SERVERPROPERTY('Edition') AS nvarchar)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Check input parameters
	---------------------------------------------------------------------------------------------------------------------------------------
	IF((@SampleInterval NOT BETWEEN 1 AND 3600) OR @SampleInterval IS NULL)
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @SampleInterval is not supported.' 
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END

	IF(@CollectingInterval NOT BETWEEN 1 AND 1440 AND @CollectingInterval IS NOT NULL)
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @CollectingInterval is not supported.' 
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END
	ELSE 
	BEGIN
		IF(@CollectingInterval IS NOT NULL AND @SampleInterval > @CollectingInterval*60 )
		BEGIN
			SET @ErrorMessage = N'The time interval of sampling is larger than the collecting time interval!'
			SET @Severity=16
			RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
		END
	END

	IF @TSMode IS NULL 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @TSMode is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @EmptyTSTable IS NULL 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @EmptyTSTable is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @Retention < 0 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @Retention is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	
	IF @help IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @help is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @LogInfo IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @LogInfo is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @debug IS NULL OR @debug < 0
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @debug is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	--------------
	-- TSMode = 0
	--------------
	IF(@TSMode = 0)
	BEGIN
		SET @DestTable = N'iovf_stats'
		SET @DestSchema = N'dbo'
	END
	--------------
	-- TSMode = 1
	--------------
	ELSE
	BEGIN
		IF(@DestTable IS NOT NULL)
		BEGIN
			SET @DestTable = REPLACE(REPLACE(RTRIM(LTRIM(@DestTable)), N'[',N''), N']',N'')
		END
		ELSE
		BEGIN
			SET @DestTAble = N'ts_iovf_stats'
		END
		
		IF(@DestSchema IS NOT NULL)
		BEGIN
			SET @DestSchema = REPLACE(REPLACE(RTRIM(LTRIM(@DestSchema)), N'[',N''), N']',N'')
		END
		ELSE
		BEGIN
			SET @DestSchema = N'dbo'
		END
	END
	
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
	SET @InfoMessage = N'Command Executed: EXEC ' + (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID)) 
		+ N' @DestTable = ' + ISNULL(N''''+@DestTable+N'''', N'NULL')
		+ N', @DestSchema  = ' + ISNULL(N''''+@DestSchema+N'''', N'NULL')
		+ N', @TSMode = ' + CAST(@TSMode AS nvarchar)
		+ N', @EmptyTSTable = ' + CAST(@EmptyTSTable AS nvarchar)
		+ N', @CollectingInterval  = ' + ISNULL(CAST(@CollectingInterval AS nvarchar), N'NULL')
		+ N', @SampleInterval = ' + CAST(@SampleInterval AS nvarchar)
		+ N', @Retention = ' + ISNULL(CAST(@Retention AS nvarchar), N'NULL')
		+ N', @LogInfo = ' + CAST(@LogInfo AS nvarchar) 
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	IF(@Retention > 0 AND @TSMode = 1)
	BEGIN
		SET @Retention = 0
		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = N'Parameter @Retention has been changed to 0. Reason: @TSMode is enabled. Tables for troubleshooting have to be maintained manually.' 
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Schema check
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(OBJECT_ID(QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable)) IS NULL)
	BEGIN
		SET @ErrorMessage = N'Table ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + N' does not exist!' 
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END

	IF(@TSMode = 1 AND @EmptyTSTable = 1)
	BEGIN
		SET @SQLCommand = N'TRUNCATE TABLE ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) 
		EXEC sp_executesql @SQLCommand
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Create temp tables and set variables
	---------------------------------------------------------------------------------------------------------------------------------------
	SET @CollectSQLCommand = N'INSERT INTO '+ QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + N' 
		 ([capture_date], [database_name], [type], [physical_name], [sample_ms], [num_of_reads], [num_of_bytes_read], [io_stall_read_ms], [num_of_writes], [num_of_bytes_written], [io_stall_write_ms]) 
		 SELECT [tio2].[capture_date], DB_NAME([tio2].[database_id]), [mf].[type], [mf].[physical_name], [tio2].[sample_ms]-[tio1].[sample_ms], [tio2].[num_of_reads]-[tio1].[num_of_reads], [tio2].[num_of_bytes_read]-[tio1].[num_of_bytes_read], [tio2].[io_stall_read_ms]-[tio1].[io_stall_read_ms], [tio2].[num_of_writes]-[tio1].[num_of_writes], [tio2].[num_of_bytes_written]-[tio1].[num_of_bytes_written], [tio2].[io_stall_write_ms]-[tio1].[io_stall_write_ms] 
		 FROM #temp_IOVFStats1 AS [tio1] INNER JOIN #temp_IOVFStats2 AS [tio2] 
		 ON [tio1].[database_id] = [tio2].[database_id] AND [tio1].[file_id] = [tio2].[file_id] 
		 INNER JOIN sys.master_files AS [mf] 
		 ON [tio2].[database_id] = [mf].[database_id] AND [tio2].[file_id] = [mf].[file_id] 
		 INNER JOIN #temp_filter AS [tf] ON [tf].[database_id] = [tio2].[database_id] AND [tf].[logical_file_id] = [tio2].[file_id] '

	CREATE TABLE #temp_IOVFStats1
		(
		[database_id] SMALLINT, 
		[file_id] SMALLINT, 
		[sample_ms] INT, 
		[num_of_reads] BIGINT,
		[num_of_bytes_read] BIGINT,
		[io_stall_read_ms] BIGINT,
		[num_of_writes] BIGINT,
		[num_of_bytes_written] BIGINT,
		[io_stall_write_ms] BIGINT
		)
	
	CREATE TABLE #temp_IOVFStats2
		(
		[capture_date] DATETIME DEFAULT GETDATE(),
		[database_id] SMALLINT, 
		[file_id] SMALLINT, 
		[sample_ms] INT, 
		[num_of_reads] BIGINT,
		[num_of_bytes_read] BIGINT,
		[io_stall_read_ms] BIGINT,
		[num_of_writes] BIGINT,
		[num_of_bytes_written] BIGINT,
		[io_stall_write_ms] BIGINT
		)

	CREATE TABLE #temp_filter
		(
		[database_id] INT, 
		[logical_file_id] INT, 
		)
	---------------------------------------------------------
	-- Filtering | TSMode - include | StandardMode - exclude
	---------------------------------------------------------
	IF(@TSMode = 1) /* INCLUDE */
	BEGIN
		INSERT INTO #temp_filter ( [database_id], [logical_file_id] )
		SELECT [iovfs].[database_id], [iovfs].[file_id] FROM sys.dm_io_virtual_file_stats(NULL, NULL) AS [iovfs]
		INNER JOIN sys.master_files AS [mf] ON [iovfs].[database_id] = [mf].[database_id] AND [iovfs].[file_id] = [mf].[file_id] 
		INNER JOIN [dbo].[filter_database_file] AS [ft] ON [ft].[database_name] = DB_NAME([iovfs].[database_id]) AND [ft].[logical_file_name] = [mf].[name]
		WHERE [ft].[ts_is_included_iovf] = 1
	END
	ELSE /* EXCLUDE */ 
	BEGIN
		INSERT INTO #temp_filter ( [database_id], [logical_file_id] )
		SELECT [iovfs].[database_id], [iovfs].[file_id] FROM sys.dm_io_virtual_file_stats(NULL, NULL) AS [iovfs]
		INNER JOIN sys.master_files AS [mf] ON [iovfs].[database_id] = [mf].[database_id] AND [iovfs].[file_id] = [mf].[file_id] 
		LEFT OUTER JOIN [dbo].[filter_database_file] AS [ft] ON [ft].[database_name] = DB_NAME([iovfs].[database_id]) AND [ft].[logical_file_name] = [mf].[name]
		WHERE [ft].[is_excluded_iovf] = 0 OR [ft].[is_excluded_iovf] IS NULL
		
	END

	

	IF(@CollectingInterval IS NULL)
	BEGIN
		SET @max_loop = 1
	END
	ELSE
	BEGIN
		SET @max_loop = @CollectingInterval*60/@SampleInterval
	END

	IF(@debug > 0)
	BEGIN
		SET @InfoMessage = N'Number of loops - @max_loop: ' + CAST(@max_loop AS nvarchar) + CHAR(13) + CHAR(10)
			+ N'Sample interval - @SampleInterval: ' + CAST(@SampleInterval AS nvarchar) 
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	SET @loop_count = 0
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Collect IO Virtual File Stats Data 
	---------------------------------------------------------------------------------------------------------------------------------------
	-----------------------------------------
	-- initial measurement
	-----------------------------------------
	INSERT INTO #temp_IOVFStats1
		([database_id], [file_id], [sample_ms], [num_of_reads], [num_of_bytes_read], [io_stall_read_ms], [num_of_writes], [num_of_bytes_written], [io_stall_write_ms])
		SELECT [database_id], [file_id], [sample_ms], [num_of_reads], [num_of_bytes_read], [io_stall_read_ms], [num_of_writes], [num_of_bytes_written], [io_stall_write_ms]
		FROM sys.dm_io_virtual_file_stats(NULL, NULL)
	
	IF(@debug > 1)
	BEGIN
		SELECT '#temp_IOVFStats1'
		SELECT * FROM #temp_IOVFStats1
	END

	SET @start_loop = GETDATE()

	WHILE(@loop_count < @max_loop)
	BEGIN
		SET @sample_interval = CONVERT(nvarchar, DATEADD(ms, @SampleInterval*1000-CAST(DATEDIFF(ms,@start_loop, GETDATE()) AS INT), 0), 114)

		WAITFOR DELAY @sample_interval;

		-----------------------------------------
		-- 2nd measurement
		-----------------------------------------
		INSERT INTO #temp_IOVFStats2
		([database_id], [file_id], [sample_ms], [num_of_reads], [num_of_bytes_read], [io_stall_read_ms], [num_of_writes], [num_of_bytes_written], [io_stall_write_ms])
		SELECT [database_id], [file_id], [sample_ms], [num_of_reads], [num_of_bytes_read], [io_stall_read_ms], [num_of_writes], [num_of_bytes_written], [io_stall_write_ms]
		FROM sys.dm_io_virtual_file_stats(NULL, NULL)
		
		IF(@debug > 1)
		BEGIN
			SELECT '#temp_IOVFStats2'
			SELECT * FROM #temp_IOVFStats2
		END

		SET @start_loop = GETDATE()
		
		------------------------------------------------------------------------------------------------
		-- Count and store the differences of the 1st and the 2nd measurement, prepare #temp_IOVFStats1
		------------------------------------------------------------------------------------------------
		EXEC sp_executesql @CollectSQLCommand

		TRUNCATE TABLE #temp_IOVFStats1

		INSERT INTO #temp_IOVFStats1 
		SELECT [database_id], [file_id], [sample_ms], [num_of_reads], [num_of_bytes_read], [io_stall_read_ms], [num_of_writes], [num_of_bytes_written], [io_stall_write_ms] 
		FROM #temp_IOVFStats2

		TRUNCATE TABLE #temp_IOVFStats2

		SET @loop_count = @loop_count + 1

	END

	DROP TABLE #temp_IOVFStats1
	
	DROP TABLE #temp_IOVFStats2

	DROP TABLE #temp_filter

	---------------------------------------------------------------------------------------------------------
	-- Delete expired data
	---------------------------------------------------------------------------------------------------------
	IF(@Retention > 0)
	BEGIN
		SET @SQLCommand = N'DELETE FROM ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + ' WHERE [capture_date] < DATEADD(dd, -1*' + CAST(@retention AS nvarchar) + N', GETDATE()) '
		EXEC sp_executesql @SQLCommand

		SET @row_count = @@ROWCOUNT

		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) deleted due to expired retention.' 				
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
	END

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = 'Outcome: Succeeded' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END



END TRY
---------------------------------------------------------------------------------------------------------------------------------------
-- Handle Errors
---------------------------------------------------------------------------------------------------------------------------------------
BEGIN CATCH
	DECLARE @ErrMsg NVARCHAR(4000)
	DECLARE @ErrSev INT
	SET @ErrSev = ERROR_SEVERITY()
	SET @ErrMsg = ERROR_MESSAGE()

	IF OBJECT_ID ('tempdb..#temp_IOVFStats1') IS NOT NULL
	BEGIN
		DROP TABLE #temp_IOVFStats1
	END

	IF OBJECT_ID ('tempdb..#temp_IOVFStats2') IS NOT NULL
	BEGIN
		DROP TABLE #temp_IOVFStats2
	END

	IF OBJECT_ID ('tempdb..#temp_filter') IS NOT NULL
	BEGIN
		DROP TABLE #temp_filter
	END
	
	RAISERROR(@ErrMsg, @ErrSev, 1) WITH NOWAIT
	
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Failed' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
END CATCH
END
GO
