USE [BaselineDB]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF OBJECT_ID('dbo.sp_CollectInstanceInfo') IS NOT NULL
    DROP PROC dbo.sp_CollectInstanceInfo;
GO

CREATE PROC [dbo].[sp_CollectInstanceInfo]

/*********************************************************************************************
Written by Robert Virag

File: sp_CollectInstanceInfo.sql

Example: 
EXEC sp_CollectInstanceInfo
@Retention = 90,
@help = 0,
@LogInfo = 1

Version: V1.2 - 2015.05

Tested: SQL Server 2005 / 2008 / 2008 R2 / 2012 / 2014 

Summary:
Collecting SQL Server Server Instace Related Info

License: Copyright (c) 2015 Robert Virag | SQLApprentice.net

The solution is free: http://www.sqlapprentice.net/license/

*********************************************************************************************/

@Retention SMALLINT = 90,
@debug TINYINT = 0,
@help BIT = 0,
@LogInfo BIT = 1

AS
BEGIN
SET NOCOUNT ON

IF(@help = 1)
BEGIN
	SELECT (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID)) AS [Collector]
	
	SELECT N'@help' AS [Parameter Name],
           N'BIT' AS [Data Type],
		   N'0' AS [Default Value],
           N'Display help messages.' AS [Parameter Description]

    UNION ALL
	SELECT N'@LogInfo', N'BIT', N'1', N'Generate Log Information'

	UNION ALL
	SELECT N'@Retention', N'SMALLINT', N'90', N'Specify the time, in days, after which the data collected are deleted. If no time is specified, then no data are deleted.'

	RETURN
END

DECLARE @ErrorMessage NVARCHAR(max)
DECLARE @InfoMessage NVARCHAR(max)
DECLARE @Severity INT

DECLARE @row_count INT
DECLARE @config_name NVARCHAR(35)

BEGIN TRY
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Start
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		+ N'Server: ' + CAST(SERVERPROPERTY('ServerName') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Version: ' + CAST(SERVERPROPERTY('ProductVersion') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Edition: ' + CAST(SERVERPROPERTY('Edition') AS nvarchar)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
		
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Check input parameters
	---------------------------------------------------------------------------------------------------------------------------------------
	IF @Retention < 0  
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @Retention is not supported.'
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	
	IF @help IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @help is not supported.'
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @LogInfo IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @LogInfo is not supported.'
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @debug IS NULL OR @debug < 0
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @debug is not supported.'
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Command
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Command Executed: EXEC ' + (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID))
		+ N' @Retention = ' + ISNULL(CAST(@Retention AS nvarchar), N'NULL')
		+ N', @LogInfo = '+ CAST(@LogInfo AS nvarchar)
		+ N', @help = ' + CAST(@help AS nvarchar) 
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Schema check
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(OBJECT_ID(N'[dbo].[instance_info]') IS NULL)
	BEGIN
		SET @ErrorMessage = N'Table [dbo].[instance_info] does not exist!'
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END
		
	SET @row_count = 0;
	--
	DECLARE @instance_info TABLE
	(
		[name] NVARCHAR(50) NOT NULL,
		[value] SQL_VARIANT NULL 
	)
	INSERT INTO @instance_info ([name], [value]) SELECT 'ComputerNamePhysicalNetBIOS', SERVERPROPERTY('ComputerNamePhysicalNetBIOS')
	INSERT INTO @instance_info ([name], [value]) SELECT 'MachineName', SERVERPROPERTY('MachineName')
	INSERT INTO @instance_info ([name], [value]) SELECT 'ServerName', SERVERPROPERTY('ServerName')
	INSERT INTO @instance_info ([name], [value]) SELECT 'InstanceName', SERVERPROPERTY('InstanceName')
	INSERT INTO @instance_info ([name], [value]) SELECT 'IsClustered', SERVERPROPERTY('IsClustered')
	INSERT INTO @instance_info ([name], [value]) SELECT 'Edition', SERVERPROPERTY('Edition')
	INSERT INTO @instance_info ([name], [value]) SELECT 'ProductLevel', SERVERPROPERTY('ProductLevel')
	INSERT INTO @instance_info ([name], [value]) SELECT 'ProductVersion', SERVERPROPERTY('ProductVersion')
	INSERT INTO @instance_info ([name], [value]) SELECT 'ResourceLastUpdateDateTime', SERVERPROPERTY('ResourceLastUpdateDateTime')
	INSERT INTO @instance_info ([name], [value]) SELECT 'ResourceVersion', SERVERPROPERTY('ResourceVersion')
	INSERT INTO @instance_info ([name], [value]) SELECT 'LicenseType', SERVERPROPERTY('LicenseType')
	INSERT INTO @instance_info ([name], [value]) SELECT 'NumLicenses', SERVERPROPERTY('NumLicenses')
	INSERT INTO @instance_info ([name], [value]) SELECT 'ProcessID', SERVERPROPERTY('ProcessID')
	INSERT INTO @instance_info ([name], [value]) SELECT 'Collation', SERVERPROPERTY('Collation')
	INSERT INTO @instance_info ([name], [value]) SELECT 'IsFullTextInstalled', SERVERPROPERTY('IsFullTextInstalled')
	INSERT INTO @instance_info ([name], [value]) SELECT 'IsIntegratedSecurityOnly', SERVERPROPERTY('IsIntegratedSecurityOnly')
	INSERT INTO @instance_info ([name], [value]) SELECT [type_desc], COUNT([type]) FROM sys .server_principals GROUP BY [type_desc]
	INSERT INTO @instance_info ([name], [value]) SELECT 'NumDatabases: ' + [state_desc] , COUNT([name]) FROM sys.databases GROUP BY [state_desc]

	--select * from @instance_info
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Pre-Initial Data: insert data which have no initial state (e.g.: table is empty or data were deleted manually since the last check)
	---------------------------------------------------------------------------------------------------------------------------------------
	;WITH most_recent_config as ( 
	SELECT [capture_date], [name]--, [Value], [value_in_use], [is_initial], [is_pre]
	FROM [dbo].[instance_info] AS [c] WHERE [capture_date] = (SELECT MAX([capture_date]) FROM [dbo].[instance_info] AS [cc] WHERE [c].[name] = [cc].[name])
	) 
	INSERT INTO [dbo].[instance_info]
    ( [capture_date], [name], [Value], [is_initial], [is_pre] ) 
	SELECT  GETDATE(),[name], [value], 1, 1 
	FROM @instance_info WHERE [name] NOT IN (SELECT [name] FROM most_recent_config) 
	
	--select * from [dbo].[instance_info]

	SET @row_count = @@ROWCOUNT

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) added as pre-initial instance info data.'
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END;

	---------------------------------------------------------------------------------------------------------
	-- Check and collect changed configuration data
	---------------------------------------------------------------------------------------------------------
	WITH [most_recent_config] AS ( 
	SELECT [capture_date], [name], [Value]
	FROM [dbo].[instance_info] AS [c] WHERE [capture_date] = (SELECT MAX([capture_date]) FROM [dbo].[instance_info] AS [cc] WHERE [c].[name] = [cc].[name])
	)
	INSERT  INTO [dbo].[instance_info] ( [capture_date], [name], [Value], [is_initial], [is_pre] )
	SELECT GETDATE(), [c].[name], [c].[value], 0, 0 FROM [most_recent_config] AS [mrc] INNER JOIN @instance_info AS [c] ON [mrc].[name] = [c].[name] WHERE [mrc].[value] <> [c].[value]

	SET @row_count = @@ROWCOUNT

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) added as changed instance info data.'
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	---------------------------------------------------------------------------------------------------------
	-- Delete expired data
	---------------------------------------------------------------------------------------------------------
	IF(@Retention > 0)
	BEGIN
		DELETE FROM [dbo].[instance_info] WHERE [capture_date] < DATEADD(dd, -1*CAST(@retention AS nvarchar) , GETDATE())

		SET @row_count = @@ROWCOUNT

		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) deleted due to expired retention.'				
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

		---------------------------------------------------------------------------------------------------------
		-- Post-Initial Data: insert data which have no initial state because of the previous expired-deletion
		---------------------------------------------------------------------------------------------------------
		INSERT INTO [dbo].[instance_info]
		( [capture_date], [name], [Value], [is_initial], [is_pre]  )
		SELECT GETDATE(), [name], [value], 1, 0 
		FROM @instance_info 
		WHERE [name] NOT IN ( 
		SELECT [name] FROM [dbo].[instance_info] 
		WHERE [capture_date] >= DATEADD(dd, -1*@retention, GETDATE())
		)

		SET @row_count = @@ROWCOUNT

		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) added as post-initial instance info data.'				
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
	END;

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Succeeded' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

END TRY
---------------------------------------------------------------------------------------------------------------------------------------
-- Handle Errors
---------------------------------------------------------------------------------------------------------------------------------------
BEGIN CATCH
	DECLARE @ErrMsg NVARCHAR(4000)
	DECLARE @ErrSev INT
	SET @ErrSev = ERROR_SEVERITY()
	SET @ErrMsg = ERROR_MESSAGE()

	RAISERROR(@ErrMsg, @ErrSev, 1) WITH NOWAIT
	
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Failed' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
END CATCH
END