USE [BaselineDB]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_NAME = 'sp_CollectPerfmonData')
	EXEC ('CREATE PROC dbo.sp_CollectPerfmonData AS SELECT ''stub version, to be replaced''')
GO

ALTER PROC [dbo].[sp_CollectPerfmonData]

/*********************************************************************************************
Written by Robert Virag

File: sp_CollectPerfmonData.sql

Example:
EXEC sp_CollectPerfmonData
@DestTable = NULL,
@DestSchema = NULL,
@TSMode = 0,
@EmptyTSTable = 0,
@CollectingInterval = NULL,
@SampleInterval = 10,
@MeasuringInterval = 3,
@Retention = 90,
@help = 0,
@LogInfo = 1

Version: V1.2 - 2015.05

Tested: SQL Server 2005 / 2008 / 2008 R2 / 2012 / 2014  

Summary:
Collecting SQL Server Instance Related Performance Counters

License: Copyright (c) 2014 Robert Virag | SQLApprentice.net

The solution is free: http://www.sqlapprentice.net/license/

*********************************************************************************************/

@DestTable NVARCHAR(128) = NULL,
@DestSchema NVARCHAR(25) = NULL,
@TSMode BIT = 0,
@EmptyTSTable BIT = 0,
@CollectingInterval SMALLINT = NULL,
@SampleInterval SMALLINT = 10,
@MeasuringInterval TINYINT = 3,
@Retention SMALLINT = 90,
@debug TINYINT = 0,
@help BIT = 0,
@LogInfo BIT = 1

AS
BEGIN
SET NOCOUNT ON

IF(@help =1)
BEGIN
	SELECT (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID)) AS [Collector]
	
	SELECT N'@help' AS [Parameter Name],
           N'BIT' AS [Data Type],
		   N'0' AS [Default Value],
           N'Display help message.' AS [Parameter Description]

    UNION ALL
	SELECT N'@LogInfo', N'BIT', N'1', N'Generate Log Information'

	UNION ALL
	SELECT N'@Retention', N'SMALLINT', N'90', N'Specify the time, in days, after which the data collected are deleted. If no time is specified, then no data are deleted.'
	
	UNION ALL
	SELECT N'@DestTable', N'NVARCHAR(128)', N'NULL', N'In troubleshooting mode it specifies the output table. Table must exist. Can be NULL.'

	UNION ALL
	SELECT N'@DestSchema', N'NVARCHAR(25)', N'NULL', N'In troubleshooting mode it specifies the schema of the output table. Schema must exist. Can be NULL.'

	UNION ALL
	SELECT N'@TSMode', N'BIT', N'0', N'Enable troubleshooting mode'

	UNION ALL
	SELECT N'@EmptyTSTable', N'BIT', N'0', N'Empty the table used for troubleshooting before the data collection.'

	UNION ALL
	SELECT N'@CollectingInterval', N'SMALLINT', N'NULL', N'Duration of the data collection in minutes. Maximum 1440 (1 day), so value has to be between 1 and 1440. It can be NULL. In that case the collector only takes one sample, the actual state of the data collected by the collector.'

	UNION ALL
	SELECT N'@SampleInterval', N'SMALLINT', N'10', N'Time in seconds between taking two samples. (1-180)'
	
	UNION ALL
	SELECT N'@MeasuringInterval', N'TINYINT', N'3', N'Time in seconds between the 1st and 2nd measurement of the data sampling for measuring data during the collecting interval. (1-60)'
	
	SELECT N'Standard' AS [Running Mode], N'[dbo].[perfmon_data]' AS [Default Table]
	UNION ALL
	SELECT N'Troubleshooting', N'[dbo].[ts_perfmon_data]'

	SELECT N'Filtering table' AS [Filtering], N'[dbo].[filter_performance_counters]' AS [Info]
    UNION ALL
	SELECT N'Type of Filtering', N'Include'
	UNION ALL
	SELECT N'Column used in Standardmode', N'[is_included]'
	UNION ALL
	SELECT N'Column used in TSMode', N'[ts_is_included]'  

	SELECT [counter_name], [is_included], [ts_is_included] FROM [dbo].[filter_performance_counters] ORDER BY [counter_name]

	RETURN
END

DECLARE @ErrorMessage NVARCHAR(max)
DECLARE @InfoMessage NVARCHAR(max)
DECLARE @Severity INT

DECLARE @FirstMeasurementSQLCommand NVARCHAR(max)
DECLARE @SecondMeasurementSQLCommand NVARCHAR(max)
DECLARE @CollectSQLCommand NVARCHAR(max)
DECLARE @SQLCommand NVARCHAR(max)
DECLARE @waitfor_delay NVARCHAR (8)
DECLARE @sample_interval NVARCHAR(12)

DECLARE @start_loop DATETIME
DECLARE @max_loop INT
DECLARE @loop_count INT

DECLARE @row_count INT

BEGIN TRY
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Start
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
			+ N'Server: ' + CAST(SERVERPROPERTY('ServerName') AS nvarchar) + CHAR(13) + CHAR(10)
			+ N'Version: ' + CAST(SERVERPROPERTY('ProductVersion') AS nvarchar) + CHAR(13) + CHAR(10)
			+ N'Edition: ' + CAST(SERVERPROPERTY('Edition') AS nvarchar)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Check input parameters
	---------------------------------------------------------------------------------------------------------------------------------------
	IF((@MeasuringInterval NOT BETWEEN 1 AND 60) OR @MeasuringInterval IS NULL)
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @MeasuringInterval is not supported.' 
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END

	IF((@CollectingInterval IS NULL AND @SampleInterval NOT BETWEEN 1 AND 180) 
		OR (@CollectingInterval IS NOT NULL AND (@SampleInterval NOT BETWEEN 1 AND 180 OR @SampleInterval IS NULL)))
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @SampleInterval is not supported.' 
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END

	IF(@CollectingInterval NOT BETWEEN 1 AND 1440 AND @CollectingInterval IS NOT NULL)
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @CollectingInterval is not supported.' 
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END
	ELSE
	BEGIN
		IF(@MeasuringInterval >= @SampleInterval)
		BEGIN
			SET @ErrorMessage = N'Parameter @SampleInterval MUST be larger than @MeasuringInterval!'
			SET @Severity=16
			RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
		END
	END

	IF @TSMode IS NULL 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @TSMode is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @EmptyTSTable IS NULL 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @EmptyTSTable is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @Retention < 0 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @Retention is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	
	IF @help IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @help is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @LogInfo IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @LogInfo is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @debug IS NULL OR @debug < 0
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @debug is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	--------------
	-- TSMode = 0
	--------------
	IF(@TSMode = 0)
	BEGIN
		SET @DestTable = N'perfmon_data'
		SET @DestSchema = N'dbo'
	END
	--------------
	-- TSMode = 1
	--------------
	ELSE
	BEGIN
		IF(@DestTable IS NOT NULL)
		BEGIN
			SET @DestTable = REPLACE(REPLACE(RTRIM(LTRIM(@DestTable)), N'[',N''), N']',N'')
		END
		ELSE
		BEGIN
			SET @DestTAble = N'ts_perfmon_data'
		END
		
		IF(@DestSchema IS NOT NULL)
		BEGIN
			SET @DestSchema = REPLACE(REPLACE(RTRIM(LTRIM(@DestSchema)), N'[',N''), N']',N'')
		END
		ELSE
		BEGIN
			SET @DestSchema = N'dbo'
		END
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Command
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Command Executed: EXEC ' + (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID)) 
			+ N' @DestTable = ' + ISNULL(N''''+@DestTable+N'''', N'NULL')
			+ N', @DestSchema  = ' + ISNULL(N''''+@DestSchema+N'''', N'NULL')
			+ N', @TSMode = ' + CAST(@TSMode AS nvarchar)
			+ N', @EmptyTSTable = ' + CAST(@EmptyTSTable AS nvarchar)
			+ N', @CollectingInterval  = ' + ISNULL(CAST(@CollectingInterval AS nvarchar), N'NULL')
			+ N', @SampleInterval = ' + CAST(@SampleInterval AS nvarchar)
			+ N', @MeasuringInterval = ' + CAST(@MeasuringInterval AS nvarchar)
			+ N', @Retention = ' + ISNULL(CAST(@Retention AS nvarchar), N'NULL')
			+ N', @LogInfo = ' + CAST(@LogInfo AS nvarchar)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	IF(@Retention > 0 AND @TSMode = 1)
	BEGIN
		SET @Retention = NULL
		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = N'Parameter @Retention has been changed to 0. Reason: @TSMode is enabled. Tables for troubleshooting have to be maintained manually.' 
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Schema check
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(OBJECT_ID(QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable)) IS NULL)
	BEGIN
		SET @ErrorMessage = N'Table ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + N' does not exist!' 
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END

	IF(@TSMode = 1 AND @EmptyTSTable = 1)
	BEGIN
		SET @SQLCommand = N'TRUNCATE TABLE ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) 
		EXEC sp_executesql @SQLCommand
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Create temp table and set variables
	---------------------------------------------------------------------------------------------------------------------------------------
	SET @FirstMeasurementSQLCommand = N'INSERT INTO #temp_perfcounters 
			( [Counter], [CounterType], [FirstValue], [FirstDateTime] ) 
			 SELECT 
			 RIGHT(RTRIM(object_name),CHARINDEX('':'',REVERSE(RTRIM(object_name)))-1) + N'':'' + RTRIM([counter_name]) + N'':'' + RTRIM([instance_name]), 
			 [cntr_type], [cntr_value], GETDATE() 
			 FROM sys.dm_os_performance_counters '
	
	IF(@TSMode = 0)
	BEGIN
		SET @FirstMeasurementSQLCommand = @FirstMeasurementSQLCommand + N' WHERE [counter_name] IN ( SELECT [counter_name] FROM [dbo].[filter_performance_counters] WHERE [is_included] = 1 ) '
	END
	ELSE
	BEGIN
		SET @FirstMeasurementSQLCommand = @FirstMeasurementSQLCommand + N' WHERE [counter_name] IN ( SELECT [counter_name] FROM [dbo].[filter_performance_counters] WHERE [ts_is_included] = 1 ) '
	END

	SET @SecondMeasurementSQLCommand = N'UPDATE #temp_perfcounters 
			 SET [SecondValue] = [cntr_value], 
			 [SecondDateTime] = GETDATE() 
			 FROM #temp_perfcounters INNER JOIN sys.dm_os_performance_counters 
			 ON [Counter] = RIGHT(RTRIM(object_name),CHARINDEX('':'',REVERSE(RTRIM(object_name)))-1) + N'':'' + RTRIM([counter_name]) + N'':'' + RTRIM([instance_name]) ' 
	
	IF(@TSMode = 0)
	BEGIN
		SET @SecondMeasurementSQLCommand = @SecondMeasurementSQLCommand + N' WHERE [counter_name] IN ( SELECT [counter_name] FROM [dbo].[filter_performance_counters] WHERE [is_included] = 1 ) '
	END
	ELSE
	BEGIN
		SET @SecondMeasurementSQLCommand = @SecondMeasurementSQLCommand + N' WHERE [counter_name] IN ( SELECT [counter_name] FROM [dbo].[filter_performance_counters] WHERE [ts_is_included] = 1 ) '
	END
		
	SET @CollectSQLCommand = N'INSERT INTO '+ QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + N' 
			 ([counter], [value], [capture_date] ) 
			 SELECT [Counter], [CounterValue], [SecondDateTime] 
			 FROM #temp_perfcounters '

	CREATE TABLE #temp_perfcounters
		(
		[Counter] NVARCHAR(770), 
		[CounterType] INT, 
		[FirstValue] DECIMAL(38, 2), 
		[FirstDateTime] DATETIME, 
		[SecondValue] DECIMAL(38, 2), 
		[SecondDateTime] DATETIME, 
		[ValueDiff] AS ( [SecondValue] - [FirstValue] ), 
		[TimeDiff] AS ( DATEDIFF(SS, FirstDateTime, SecondDateTime) ), 
		[CounterValue] DECIMAL(38, 2)
		)

	IF(@CollectingInterval IS NULL)
	BEGIN
		SET @max_loop = 1
		SET @SampleInterval = NULL
	END
	ELSE
	BEGIN
		SET @max_loop = @CollectingInterval*60/@SampleInterval
	END

	SET @waitfor_delay = CONVERT(NVARCHAR(8), DATEADD(ss, @MeasuringInterval, 0), 114)

	IF(@debug > 0)
	BEGIN
		SET @InfoMessage = N'Number of loops - @max_loop: ' + CAST(@max_loop AS nvarchar) + CHAR(13) + CHAR(10)
			+ N'Time between the 1st and 2nd measurement - @waitfor_delay: ' + @waitfor_delay 
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	SET @loop_count = 0

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Collect performance counters
	---------------------------------------------------------------------------------------------------------------------------------------
	WHILE(@loop_count < @max_loop)
	BEGIN
		SET @start_loop = GETDATE()
		-----------------------------------------
		-- 1st measurement
		-----------------------------------------
		IF(@debug > 0)
		BEGIN
			SET @InfoMessage = N'FirstMeasurement~' + CONVERT(nvarchar, @start_loop,21) 
			IF(@debug > 1)
			BEGIN
				IF(@debug > 2)
				BEGIN
					SELECT @InfoMessage
					SELECT * FROM  #temp_perfcounters
				END
				SET @InfoMessage = @InfoMessage + CHAR(13) + CHAR(10) + @FirstMeasurementSQLCommand
			END
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

		EXEC sp_executesql @FirstMeasurementSQLCommand

		WAITFOR DELAY @waitfor_delay;

		-----------------------------------------
		-- 2nd measurement
		-----------------------------------------
		IF(@debug > 0)
		BEGIN
			SET @InfoMessage = N'SecondMeasurement~' + CONVERT(nvarchar, GETDATE(),21) + N' /'+ CONVERT(nvarchar, DATEADD(ms, CAST(DATEDIFF(ms,@start_loop,GETDATE()) AS INT), 0), 114) + N'/' 
			IF(@debug > 1)
			BEGIN
				IF(@debug > 2)
				BEGIN
					SELECT @InfoMessage
					SELECT * FROM #temp_perfcounters
				END
				SET @InfoMessage = @InfoMessage + CHAR(13) + CHAR(10) + @SecondMeasurementSQLCommand 
			END
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

		EXEC sp_executesql @SecondMeasurementSQLCommand
		
		-----------------------------------------
		-- Update cntr type 272696576
		-----------------------------------------
		UPDATE  #temp_perfcounters 
			SET [CounterValue] = [ValueDiff] / [TimeDiff]
			WHERE [CounterType] = 272696576

		IF(@debug > 1)
		BEGIN
			SET @InfoMessage = N'After updating cntr_type(272696576)' 
			IF(@debug > 2)
			BEGIN
				SELECT @InfoMessage
				SELECT * FROM #temp_perfcounters
			END
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

		-----------------------------------------
		-- Update cntr type 65792
		-----------------------------------------
		UPDATE #temp_perfcounters 
			SET [CounterValue] = [SecondValue]
			WHERE [CounterType] = 65792

		IF(@debug > 1)
		BEGIN
			SET @InfoMessage = N'After updating cntr_type(65792)'  
			IF(@debug > 2)
			BEGIN
				SELECT @InfoMessage
				SELECT * FROM #temp_perfcounters
			END
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

		----------------------------------------------------------------------
		-- Insert collected data to table 
		----------------------------------------------------------------------
		IF(@debug > 1)
		BEGIN
			SET @InfoMessage = N'Insert Command' + CHAR(13) + CHAR(10) + @CollectSQLCommand 
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

		EXEC sp_executesql @CollectSQLCommand

		TRUNCATE TABLE #temp_perfcounters

		SET @loop_count = @loop_count + 1

		SET @sample_interval = CONVERT(nvarchar, ISNULL(DATEADD(ms, @SampleInterval*1000-CAST(DATEDIFF(ms,@start_loop, GETDATE()) AS INT), 0), N'00:00:00'), 114)

		IF(@debug > 0)
		BEGIN
			SET @InfoMessage = N'Sample interval (inside the loop): ' + @sample_interval 
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

		WAITFOR DELAY @sample_interval;
	END

	DROP TABLE #temp_perfcounters
	
	---------------------------------------------------------------------------------------------------------
	-- Delete expired data
	---------------------------------------------------------------------------------------------------------
	IF(@Retention > 0)
	BEGIN
		SET @SQLCommand = N'DELETE FROM ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + ' WHERE [capture_date] < DATEADD(dd, -1*' + CAST(@retention AS nvarchar) + N', GETDATE()) '
		EXEC sp_executesql @SQLCommand

		SET @row_count = @@ROWCOUNT

		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) deleted due to expired retention.' 				
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
	END

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Succeeded' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
END TRY
---------------------------------------------------------------------------------------------------------------------------------------
-- Handle Errors
---------------------------------------------------------------------------------------------------------------------------------------
BEGIN CATCH
	DECLARE @ErrMsg NVARCHAR(4000)
	DECLARE @ErrSev INT
	SET @ErrSev = ERROR_SEVERITY()
	SET @ErrMsg = ERROR_MESSAGE()
	
	IF OBJECT_ID ('tempdb..#temp_perfcounters') IS NOT NULL
	BEGIN
		DROP TABLE #temp_perfcounters
	END

	RAISERROR(@ErrMsg, @ErrSev, 1) WITH NOWAIT
	
	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Failed' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
END CATCH
END
GO
