USE [BaselineDB]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE ROUTINE_NAME = 'sp_CollectWaitStats')
	EXEC ('CREATE PROC dbo.sp_CollectWaitStats AS SELECT ''stub version, to be replaced''')
GO

ALTER PROC [dbo].[sp_CollectWaitStats]

/*********************************************************************************************
Written by Robert Virag

File: sp_CollectWaitStats.sql

Example:
EXEC sp_CollectWaitStats
@DestTable = NULL,
@DestSchema = NULL,
@MeasuringInterval = NULL,
@ResetWaitStats = 0,
@TSMode = 0,
@EmptyTSTable = 0,
@Retention = 90,
@help = 0,
@LogInfo = 1

Version: V1.2 - 2015.05

Tested: SQL Server 2005 / 2008 / 2008 R2 / 2012 / 2014 

Summary:
Collect SQL Server Wait Statistics Raw Data

License: Copyright (c) 2014 Robert Virag | SQLApprentice.net

The solution is free: http://www.sqlapprentice.net/license/

*********************************************************************************************/

@DestTable NVARCHAR(128) = NULL,
@DestSchema NVARCHAR(25) = NULL,
@MeasuringInterval SMALLINT = NULL,
@ResetWaitStats BIT = 0,
@TSMode BIT = 0,
@EmptyTSTable BIT = 0,
@Retention SMALLINT = 90,
@debug TINYINT = 0,
@help BIT = 0,
@LogInfo BIT = 1

AS
BEGIN
SET NOCOUNT ON

IF(@help = 1)
BEGIN
	SELECT (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID)) AS [Collector]
	
	SELECT N'@help' AS [Parameter Name],
           N'BIT' AS [Data Type],
		   N'0' AS [Default Value],
           N'Display help message.' AS [Parameter Description]

    UNION ALL
	SELECT N'@LogInfo', N'BIT', N'1', N'Generate Log Information'

	UNION ALL
	SELECT N'@Retention', N'SMALLINT', N'90', N'Specify the time, in days, after which the data collected are deleted. If no time is specified, then no data are deleted.'
	
	UNION ALL
	SELECT N'@DestTable', N'NVARCHAR(128)', N'NULL', N'In troubleshooting mode it specifies the output table. Table must exist. Can be NULL.'

	UNION ALL
	SELECT N'@DestSchema', N'NVARCHAR(25)', N'NULL', N'In troubleshooting mode it specifies the schema of the output table. Schema must exist. Can be NULL.'

	UNION ALL
	SELECT N'@TSMode', N'BIT', N'0', N'Enable troubleshooting mode'

	UNION ALL
	SELECT N'@EmptyTSTable', N'BIT', N'0', N'Empty the table used for troubleshooting before the data collection.'

	UNION ALL
	SELECT N'@MeasuringInterval', N'SMALLINT', N'NULL', N'Time in minutes between the 1st and 2nd measurement of the data sampling for measuring data.'

	UNION ALL
	SELECT N'@ResetWaitStats', N'BIT', N'0', N'Clear the sys.dm_os_wait_stats after the data collection.'

	SELECT N'Standard' AS [Running Mode], N'[dbo].[wait_stats]' AS [Default Table]
	UNION ALL
	SELECT N'Troubleshooting', N'[dbo].[ts_wait_stats]'

	SELECT N'Filtering table' AS [Filtering], N'[dbo].[filter_wait_types]' AS [Info]
    UNION ALL
	SELECT N'Type of Filtering', N'Exclude'
	UNION ALL
	SELECT N'Column used in Standardmode', N'[is_excluded]'
	UNION ALL
	SELECT N'Column used in TSMode', N'[ts_is_excluded]'  

	SELECT [wait_type], [is_excluded], [ts_is_excluded] FROM [dbo].[filter_wait_types] ORDER BY [wait_type]

	RETURN
END

DECLARE @ErrorMessage NVARCHAR(max)
DECLARE @InfoMessage NVARCHAR(max)
DECLARE @Severity INT

DECLARE @data_compression NVARCHAR(35)
DECLARE @SQLCommand NVARCHAR(max)
DECLARE @waitfor_delay NVARCHAR (8)

DECLARE @AI_SQLTRACE_INCREMENTAL_FLUSH_SLEEP NVARCHAR(max)

DECLARE @LockResult INT
DECLARE @RequiredAppLock NVARCHAR(25)

DECLARE @row_count INT

BEGIN TRY
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info - Start
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
	SET @InfoMessage = N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		+ N'Server: ' + CAST(SERVERPROPERTY('ServerName') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Version: ' + CAST(SERVERPROPERTY('ProductVersion') AS nvarchar) + CHAR(13) + CHAR(10)
		+ N'Edition: ' + CAST(SERVERPROPERTY('Edition') AS nvarchar)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Check input parameters
	---------------------------------------------------------------------------------------------------------------------------------------
	
	IF(@MeasuringInterval NOT BETWEEN 1 AND 1440 AND @MeasuringInterval IS NOT NULL)
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @MeasuringInterval is not supported.' 
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END

	IF @ResetWaitStats IS NULL 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @ResetWaitStats is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @TSMode IS NULL 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @TSMode is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @EmptyTSTable IS NULL 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @EmptyTSTable is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @Retention < 0 
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @Retention is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	
	IF @help IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @help is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @LogInfo IS NULL
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @LogInfo is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END

	IF @debug IS NULL OR @debug < 0
	BEGIN
		SET @ErrorMessage = 'The value for the parameter @debug is not supported.' 
		RAISERROR(@ErrorMessage,16,1) WITH NOWAIT
	END
	--------------
	-- TSMode = 0
	--------------
	IF(@TSMode = 0)
	BEGIN
		SET @DestTable = N'wait_stats'
		SET @DestSchema = N'dbo'
	END
	--------------
	-- TSMode = 1
	--------------
	ELSE
	BEGIN
		IF(@DestTable IS NOT NULL)
		BEGIN
			SET @DestTable = REPLACE(REPLACE(RTRIM(LTRIM(@DestTable)), N'[',N''), N']',N'')
		END
		ELSE
		BEGIN
			SET @DestTAble = N'ts_wait_stats'
		END
		
		IF(@DestSchema IS NOT NULL)
		BEGIN
			SET @DestSchema = REPLACE(REPLACE(RTRIM(LTRIM(@DestSchema)), N'[',N''), N']',N'')
		END
		ELSE
		BEGIN
			SET @DestSchema = N'dbo'
		END
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Log Info
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(@LogInfo = 1)
	BEGIN
	SET @InfoMessage = N'Command Executed: EXEC ' + (SELECT QUOTENAME(schemas.name) FROM sys.schemas schemas INNER JOIN sys.objects objects ON schemas.[schema_id] = objects.[schema_id] WHERE [object_id] = @@PROCID) + N'.' + QUOTENAME(OBJECT_NAME(@@PROCID)) 
		+ N' @DestTable = ' + ISNULL(N''''+@DestTable+N'''', N'NULL')
		+ N', @DestSchema  = ' + ISNULL(N''''+@DestSchema+N'''', N'NULL')
		+ N', @TSMode = ' + CAST(@TSMode AS nvarchar)
		+ N', @EmptyTSTable = ' + CAST(@EmptyTSTable AS nvarchar)
		+ N', @MeasuringInterval = ' + ISNULL(CAST(@MeasuringInterval as nvarchar), N'NULL')
		+ N', @ResetWaitStats  = ' + CAST(@ResetWaitStats AS nvarchar)
		+ N', @Retention = ' + ISNULL(CAST(@Retention AS nvarchar), N'NULL')
		+ N', @LogInfo = ' + CAST(@LogInfo AS nvarchar)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	IF(@MeasuringInterval BETWEEN 1 AND 1440 AND @MeasuringInterval IS NOT NULL)
	BEGIN
		IF(@ResetWaitStats = 1)
		BEGIN
			SET @ResetWaitStats = 0
			IF(@LogInfo = 1)
			BEGIN
				SET @InfoMessage = N'Parameter @ResetWaitStats has been changed from 1 to 0. Reason: @MeasuringInterval is specified.' 
				SET @Severity = 10
				RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
			END
		END
	END

	IF(@TSMode = 1)
		BEGIN
		IF(@Retention > 0)
		BEGIN
			SET @Retention = 0
			IF(@LogInfo = 1)
			BEGIN
				SET @InfoMessage = N'Parameter @Retention has been changed to 0. Reason: @TSMode is enabled. Tables for troubleshooting have to be maintained manually.' 
				SET @Severity = 10
				RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
			END
		END
	END
		
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Schema check
	---------------------------------------------------------------------------------------------------------------------------------------
	IF(OBJECT_ID(QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable)) IS NULL)
	BEGIN
		SET @ErrorMessage = N'Table ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + N' does not exist!' 
		SET @Severity=16
		RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
	END
	---------------------------------------------------------------------------------------------------------------------------------------
	-- Handle concurrent running
	---------------------------------------------------------------------------------------------------------------------------------------	
	IF(@ResetWaitStats = 1)
	BEGIN
		SET @RequiredAppLock = N'Exclusive'
		EXEC @LockResult = sp_getapplock @Resource= N'WaitStatsData', 
			@LockMode=@RequiredAppLock, 
			@LockOwner = N'Session',
			@LockTimeout=500
		IF(@LockResult NOT IN (0,1))
		BEGIN
			SET @ErrorMessage = N'Unable to acquire the appropriate Application Lock(' + @RequiredAppLock + '). Possible cause: Another instance of sp_CollectWaitStatsData is running. sp_CollectWaitStatsData can not run parralel itself with parameter @ResetWaitStats = 1.'
			SET @Severity = 16
			RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
		END
	END
	ELSE
	BEGIN
		SET @RequiredAppLock = N'Shared'
		EXEC @LockResult = sp_getapplock @Resource= N'WaitStatsData', 
			@LockMode=@RequiredAppLock, 
			@LockOwner = N'Session',
			@LockTimeout=500
		IF(@LockResult NOT IN (0,1))
		BEGIN
			SET @ErrorMessage = N'Unable to acquire the appropriate Application Lock(' + @RequiredAppLock + '). Possible cause: Another instance of sp_CollectWaitStatsData is running with parameter @ResetWaitStats = 1. sp_CollectWaitStatsData can not run parralel itself with @ResetWaitStats = 1.'
			SET @Severity = 16
			RAISERROR(@ErrorMessage, @Severity, 1) WITH NOWAIT
			RETURN
		END
	END

	IF(@debug >1)
	BEGIN
		SET @InfoMessage = N'Acquired AppLock: ' + APPLOCK_MODE(N'public', N'WaitStatsData', N'Session') + N' | LockResult: '+ CAST(@LockResult AS NVARCHAR(5)) 
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

	---------------------------------------------------------------------------------------------------------------------------------------
	-- Collect waitstats
	---------------------------------------------------------------------------------------------------------------------------------------	
	------------------------------------------
	-- One time measurement
	------------------------------------------
	IF(@MeasuringInterval IS NULL)
	BEGIN
		SET @SQLCommand = N'INSERT INTO ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + N' 
				 ([capture_date], [wait_type], [waiting_tasks_count], [wait_time_ms], [max_wait_time_ms], [signal_wait_time_ms])  
				 SELECT GETDATE(), [wait_type], [waiting_tasks_count], [wait_time_ms], [max_wait_time_ms], [signal_wait_time_ms] 
				 FROM sys.dm_os_wait_stats '
		------------------------------------------
		-- Standard Mode
		------------------------------------------
		IF(@TSMode = 0)
		BEGIN
			SET @SQLCommand = @SQLCommand + N' WHERE [wait_type] NOT IN( SELECT [wait_type] FROM [filter_wait_types] WHERE [is_excluded] = 1 ) AND [waiting_tasks_count] <> 0'
		END
		----------------------------------------------
		-- Troubleshooting Mode
		----------------------------------------------
		ELSE
		BEGIN
			SET @SQLCommand = @SQLCommand + N' WHERE [wait_type] NOT IN( SELECT [wait_type] FROM [filter_wait_types] WHERE [ts_is_excluded] = 1 ) AND [waiting_tasks_count] <> 0 '
		END

		EXEC sp_executesql @SQLCommand
	END
	------------------------------------------
	-- Interval measurement
	------------------------------------------
	ELSE
	BEGIN
		IF(@MeasuringInterval = 1440)
		BEGIN
			SET @waitfor_delay = CONVERT(varchar, DATEADD(ss, @MeasuringInterval*60-1, 0), 114)
		END
		ELSE
		BEGIN
			SET @waitfor_delay = CONVERT(NVARCHAR(8), DATEADD(ss, @MeasuringInterval*60, 0), 114)
		END
		
		------------------------------------------
		-- 1st measurement of the interval
		------------------------------------------
		SELECT GETDATE() AS [first_measurement], [wait_type], [waiting_tasks_count], [wait_time_ms], [max_wait_time_ms], [signal_wait_time_ms] 
			INTO #temp_stats1 FROM sys.dm_os_wait_stats
	
		IF(@debug > 0)
		BEGIN
			SET @InfoMessage = N'@MeasuringInterval: ' + CAST(@MeasuringInterval AS NVARCHAR(10))+ CHAR(13) + CHAR(10)
				+N'@waitfor_delay: ' + @waitfor_delay 
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

		WAITFOR DELAY @waitfor_delay

		------------------------------------------
		-- 2nd measurement of the interval
		------------------------------------------
		SELECT GETDATE() AS [second_measurement], [wait_type], [waiting_tasks_count], [wait_time_ms], [max_wait_time_ms], [signal_wait_time_ms] 
			INTO #temp_stats2 FROM sys.dm_os_wait_stats
		
		----------------------------------------------------------------------
		-- Count and store the differences of the 1st and the 2nd measurement 
		----------------------------------------------------------------------
		IF (@@VERSION LIKE N'%Microsoft SQL Server 2005%' 
			OR (@@VERSION LIKE N'%Microsoft SQL Server 2008%' 
				AND @@VERSION NOT LIKE N'%Microsoft SQL Server 2008 R2%')) 
			AND (
			(@TSMode = 0 AND (SELECT [is_excluded] FROM [dbo].[filter_wait_types] WHERE [wait_type] = 'AI_SQLTRACE_INCREMENTAL_FLUSH_SLEEP') = 0) 
			OR
			(@TSMode = 1 AND (SELECT [ts_is_excluded] FROM [dbo].[filter_wait_types] WHERE [wait_type] = 'AI_SQLTRACE_INCREMENTAL_FLUSH_SLEEP') = 0) )
		BEGIN
			SET @AI_SQLTRACE_INCREMENTAL_FLUSH_SLEEP = N' UNION
				 SELECT GETDATE(), ''AI_SQLTRACE_INCREMENTAL_FLUSH_SLEEP'' AS [wait_type], 
				 1 AS [waiting_tasks_count], 
				 DATEDIFF(ms,(SELECT TOP 1 [first_measurement] FROM #temp_stats1),(SELECT TOP 1 [second_measurement] FROM #temp_stats2)) AS [wait_time_ms], 
				 1 AS [max_wait_time_ms], 
				 0 AS [signal_wait_time_ms] '
		END
		ELSE
		BEGIN
			SET @AI_SQLTRACE_INCREMENTAL_FLUSH_SLEEP = N''
		END

		SET @SQLCommand = N'WITH [DiffWaits] AS	(  
			/* Waits that werent in the first snapshot */
				 SELECT [ts2].[wait_type], [ts2].[waiting_tasks_count], [ts2].[wait_time_ms], [ts2].[max_wait_time_ms], [ts2].[signal_wait_time_ms] 
				 FROM #temp_stats2 AS [ts2] LEFT OUTER JOIN #temp_stats1 AS [ts1] 
				 ON [ts2].[wait_type] = [ts1].[wait_type] 
				 WHERE [ts1].[wait_type] IS NULL AND [ts2].[wait_time_ms] > 0 
				 UNION 
			/* Diff of waits in both snapshots */
				 SELECT [ts2].[wait_type], [ts2].[waiting_tasks_count] - [ts1].[waiting_tasks_count] AS [waiting_tasks_count], [ts2].[wait_time_ms] - [ts1].[wait_time_ms] AS [wait_time_ms], ts2.[max_wait_time_ms], [ts2].[signal_wait_time_ms] - [ts1].[signal_wait_time_ms] AS [signal_wait_time_ms] 
				 FROM #temp_stats2 AS [ts2] LEFT OUTER JOIN #temp_stats1 AS [ts1] 
				 ON [ts2].[wait_type] = [ts1].[wait_type] 
				 WHERE [ts1].[wait_type] IS NOT NULL AND [ts2].[waiting_tasks_count] - [ts1].[waiting_tasks_count] > 0 AND [ts2].[wait_time_ms] - [ts1].[wait_time_ms] > 0 
				 ) 
				 INSERT INTO ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + N' 
				 ([capture_date], [wait_type], [waiting_tasks_count], [wait_time_ms], [max_wait_time_ms], [signal_wait_time_ms])  
				 SELECT GETDATE(), [wait_type], [waiting_tasks_count], [wait_time_ms], [max_wait_time_ms], [signal_wait_time_ms] 
				 FROM DiffWaits '
		------------------------------------------
		-- Standard Mode
		------------------------------------------
		IF(@TSMode = 0)
		BEGIN
			SET @SQLCommand = @SQLCommand + N' WHERE [wait_type] NOT IN( SELECT [wait_type] FROM [filter_wait_types] WHERE [is_excluded] = 1 ) AND [waiting_tasks_count] <> 0 ' + @AI_SQLTRACE_INCREMENTAL_FLUSH_SLEEP
		END
		----------------------------------------------
		-- Troubleshooting Mode
		----------------------------------------------
		ELSE
		BEGIN
			SET @SQLCommand = @SQLCommand + N' WHERE [wait_type] NOT IN( SELECT [wait_type] FROM [filter_wait_types] WHERE [ts_is_excluded] = 1 ) AND [waiting_tasks_count] <> 0 ' + @AI_SQLTRACE_INCREMENTAL_FLUSH_SLEEP
		END 

		IF(@debug > 2)
		BEGIN
			SELECT N'#temp_stats1 Table'
			SELECT * FROM #temp_stats1

			SELECT N'#temp_stats2 Table'
			SELECT * FROM #temp_stats2

			SET @InfoMessage = N'WaitStats collector command: ' + CHAR(13) + CHAR(10) + @SQLCommand 
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

		EXEC sp_executesql @SQLCommand

		DROP TABLE #temp_stats1
		DROP TABLE #temp_stats2

	END

	------------------------------------------
	-- Reset WaitStats
	------------------------------------------
	IF(@ResetWaitStats = 1)
	BEGIN
		DBCC SQLPERF ('sys.dm_os_wait_stats', CLEAR)

		If(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = N'DMV [sys].[dm_os_wait_stats] has been reseted.' 
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
	END

	EXEC @LockResult = sp_releaseapplock 
			@Resource= N'WaitStatsData',
			@LockOwner = N'Session'

	IF(@debug > 1)
		BEGIN
			SET @InfoMessage = N'AppLock released: ' + APPLOCK_MODE(N'public', N'WaitStatsData', N'Session') + N' | LockResult: '+ CAST(@LockResult AS NVARCHAR(5)) 
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END

	---------------------------------------------------------------------------------------------------------
	-- Delete expired data
	---------------------------------------------------------------------------------------------------------
	IF(@Retention > 0)
	BEGIN
		SET @SQLCommand = N'DELETE FROM ' + QUOTENAME(@DestSchema)+N'.'+QUOTENAME(@DestTable) + ' WHERE [capture_date] < DATEADD(dd, -1*' + CAST(@retention AS nvarchar) + N', GETDATE()) '
		EXEC sp_executesql @SQLCommand

		SET @row_count = @@ROWCOUNT

		IF(@LogInfo = 1)
		BEGIN
			SET @InfoMessage = CAST(@row_count AS NVARCHAR) +  N'row(s) deleted due to expired retention.' 				
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
	END

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Succeeded' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END

END TRY
---------------------------------------------------------------------------------------------------------------------------------------
-- Handle Errors
---------------------------------------------------------------------------------------------------------------------------------------
BEGIN CATCH
	DECLARE @ErrMsg NVARCHAR(4000)
	DECLARE @ErrSev INT
	SET @ErrSev = ERROR_SEVERITY()
	SET @ErrMsg = ERROR_MESSAGE()

	IF OBJECT_ID (N'tempdb..#temp_stats1' ) IS NOT NULL
	BEGIN
		DROP TABLE #temp_stats1
	END

	IF OBJECT_ID (N'tempdb..#temp_stats2' ) IS NOT NULL
	BEGIN
		DROP TABLE #temp_stats2
	END
		
	RAISERROR(@ErrMsg, @ErrSev, 1) WITH NOWAIT

	IF(@LockResult IN (0,1))
	BEGIN
		IF(@debug > 1)
		BEGIN
			SET @InfoMessage = N'ERROR: Release AppLock: ' + APPLOCK_MODE(N'public', N'WaitStatsData', N'Session')  
			SET @Severity = 10
			RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
		END
		EXEC @LockResult = sp_releaseapplock 
			@Resource= N'WaitStatsData',
			@LockOwner = N'Session'
	END

	IF(@LogInfo = 1)
	BEGIN
		SET @InfoMessage = N'Outcome: Failed' + CHAR(13) + CHAR(10)
		+ N'Date and time: ' + CONVERT(nvarchar,GETDATE(),120) + CHAR(13) + CHAR(10)
		SET @Severity = 10
		RAISERROR(@InfoMessage, @Severity, 1) WITH NOWAIT
	END
END CATCH
END
GO
